/*********************************************************************NVMH2****
Path:  C:\Dev\devrel\Nv_sdk_4\Dx8\CommonSrc
File:  FullCoverageQuad.cpp

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/


#include "FullCoverageQuad.h"

#include <windows.h>
#include <d3d8.h>
#include <assert.h>

#include "NV_Common.h"




/////////////////////////////////////////////////////////////


FullCoverageQuad::FullCoverageQuad()
{
	m_pD3DDev = NULL;
	m_pVertexBuffer = NULL;
}


FullCoverageQuad::~FullCoverageQuad()
{
	Free();
}

/////////////////////////////////////////////////////////////


HRESULT FullCoverageQuad::Free()
{	
	SAFE_RELEASE( m_pD3DDev );
	SAFE_RELEASE( m_pVertexBuffer );

	return( S_OK );
}


HRESULT FullCoverageQuad::Initialize( LPDIRECT3DDEVICE8 pD3DDev )
{
	HRESULT hr;
	int i;

	// get the device
	assert( pD3DDev != NULL );

	m_pD3DDev = pD3DDev;
	m_pD3DDev->AddRef();			// released on Free()

	////////////////////////////////
	// create vertex buffer 

	hr = m_pD3DDev->CreateVertexBuffer( 4 * sizeof(QuadVertex), D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC, 0, D3DPOOL_DEFAULT, &m_pVertexBuffer);
	ASSERT_IF_FAILED(hr);
	if (FAILED(hr))
		return hr;

	QuadVertex      *pBuff;

	if (m_pVertexBuffer)
	{
		hr = m_pVertexBuffer->Lock(0, 4 * sizeof(QuadVertex),(BYTE**)&pBuff, 0);

		ASSERT_IF_FAILED(hr);

		if (FAILED(hr))
		{
			return hr;
		}


		float uv_base;
		float uv_max;

		uv_base = 0.0f;
		uv_max  = 1.0f; 

        for (i = 0; i < 4; ++i)
        {
            pBuff->Position = D3DXVECTOR3((i==0 || i==3) ? -1.0f : 1.0f,
                                          (i<2)          ? -1.0f : 1.0f,
                                          0.0f);
		    pBuff->Tex      = D3DXVECTOR2((i==0 || i==3) ? uv_base : uv_max , 
                                          (i<2)          ? uv_max : uv_base );
		    pBuff++;
        }
        m_pVertexBuffer->Unlock();
    }


	/////////////////////////////////////////////////
	// Set matrices so the quad created above will 
	//  exactly cover the render target

	D3DXMATRIX matView, matProj, matViewProj, matWorld;

	D3DXVECTOR3 vEyePt    = D3DXVECTOR3( 0.0f, 0.0f, -5.0f );
	D3DXVECTOR3 vLookatPt = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 vUp       = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

	// Set World, View, Projection, and combination matrices.
	D3DXMatrixLookAtLH(&matView, &vEyePt, &vLookatPt, &vUp);
	D3DXMatrixOrthoLH(&matProj, 4.0f, 4.0f, 0.2f, 20.0f);

    D3DXMatrixMultiply(&matViewProj, &matView, &matProj);

    // draw a single quad to texture:
    // the quad covers the whole "screen" exactly

	D3DXMatrixScaling(  &matWorld,		   2.0f, 2.0f, 1.0f);
	D3DXMatrixMultiply( &m_mWorldViewProj, &matWorld, &matViewProj);

	//////////////////////////////////////////////////
	// Use identity matrix for world & view matrices

	D3DXMatrixIdentity( & m_mIdentity );


	return( hr );
}



void  FullCoverageQuad::SetViewMatrices()
{
	assert( m_pD3DDev != NULL );

	m_pD3DDev->SetTransform(D3DTS_PROJECTION,	&m_mWorldViewProj	);
	m_pD3DDev->SetTransform(D3DTS_VIEW,			&m_mIdentity		);
	m_pD3DDev->SetTransform(D3DTS_WORLD,		&m_mIdentity		);

}


HRESULT FullCoverageQuad::Draw()
{
	HRESULT hr;

	assert( m_pD3DDev != NULL );
	
	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof(QuadVertex));
	ASSERT_IF_FAILED(hr);

	hr = m_pD3DDev->SetPixelShader( 0 );
	ASSERT_IF_FAILED( hr );
	hr = m_pD3DDev->SetVertexShader( D3DFVF_XYZ | D3DFVF_TEX1 | D3DFVF_TEXCOORDSIZE2(0) );
	ASSERT_IF_FAILED( hr );

	SetViewMatrices();

	hr = m_pD3DDev->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
	ASSERT_IF_FAILED( hr );

	return( hr );
}




