/*********************************************************************NVMH2****
Path:  C:\Dev\devrel\Nv_sdk_4\Dx8\CommonSrc
File:  ShadowVolumeObject8.cpp

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:
See header for comments

******************************************************************************/


#include "ShadowVolumeObject8.h"

#include "NV_Common.h"
#include "NV_Error.h"
#include <vector>
#include <algorithm>
#include <assert.h>

using namespace std;

typedef std::vector< int >	V_INT;


///////////////////////////////////////////////////////////

ShadowVolumeObject8::ShadowVolumeObject8()
{
	::WingedEdgeObject8();


	m_wOriginalModelTriIndices	= 0;
	m_wLastTriIndex				= 0;


}


ShadowVolumeObject8::~ShadowVolumeObject8()
{
	Free();
}

////////////////////////////////////////////////////////////


HRESULT ShadowVolumeObject8::Free()
{

	return( S_OK );
}


void ShadowVolumeObject8::PostInit()
{
	SimpleObject8::PostInit();

	assert( m_wNumInd > 0 );
	assert( m_wNumVerts > 0 );

	m_wLastTriIndex = m_wNumInd;
	m_wOriginalModelTriIndices = m_wNumInd;

	m_bIsValid = true;

}


void	ShadowVolumeObject8::GetTriCounts( WORD * pNumTotalTris, WORD * pNumWithoutShadowVolTris )
{
	if( pNumTotalTris != NULL )
	{
		*pNumTotalTris = m_wLastTriIndex;
	}
	if( pNumWithoutShadowVolTris != NULL )
	{
		*pNumWithoutShadowVolTris = m_wOriginalModelTriIndices;
	}
}


void ShadowVolumeObject8::ExplodeFaces( float distance )
{
	// move faces out along vector from
	//  center of object
	
	m_wNumInd = m_wOriginalModelTriIndices;

	SimpleObject8::ExplodeFaces( distance );

	m_wNumInd = m_wLastTriIndex;	
}


void	ShadowVolumeObject8::DrawShadowVolume( LPDIRECT3DDEVICE8  pDev )
{
	assert( pDev != NULL );
	assert( m_wLastTriIndex > 0 );
	assert( m_wOriginalModelTriIndices > 0 );

	FDebug("ShadowVolumeObject8::  Non-VertexBuffer rendering!  This will be slow!\n");

	pDev->DrawIndexedPrimitiveUP(	m_PrimType,
									0,					// min index
									m_wNumVerts,
									m_wLastTriIndex/3,	// prim count
									m_pIndices,
									D3DFMT_INDEX16,
									m_pVertices,
									sizeof( SimpleObjVertex ) );
}



HRESULT		ShadowVolumeObject8::BuildShadowVolume( float weld_edge_threshold )
{
	assert( m_bIsValid );
	assert( m_PrimType == D3DPT_TRIANGLELIST );
	assert( sizeof(WORD) == 2 );

	assert( m_bOpenHull == false );

	HRESULT hr = S_OK;

	const float limit = 0.9f;

	if( weld_edge_threshold > limit )
	{
		FDebug("\n");
		FDebug("########################################################\n");
		FDebug("ShadowVolume BuildShadowVolume:\n");
		FDebug("weld_edge_threshold > %f\n", limit );
		FDebug("This will result in MANY additional triangles in areas where\n");
		FDebug("   they are not needed.  Try using a threshold closer to 0.6\n");
		FDebug("########################################################\n");
		FDebug("\n");
	}	


	hr = BuildWingedEdgeData();
	ASSERT_IF_FAILED(hr);

	MakeFaceted();		// Create new vertices.  No new triangles

						// Eliminate some vertices to join edges 
						//   where the face normals are similar.
						// Thresh is in the range [-1,1]
						// A value of 1.0 will weld only faces
						//  with the same normal.
	WeldFacetedFaces( weld_edge_threshold );


	/////////////////////////////////////////////////////////////
	// For each non-welded edge (each edge with different vertices
	//  at the same spatial position), create zero-area triangles
	//  to close the edge.
	
//	assert( IsWingedEdgeDataBuilt() );

	// rebuild the winged-edge data now that some faces
	// have been welded

	BuildWingedEdgeData();

	int		tri;
	int		n, i;

	WORD	last_edge_vertex;
	WORD	neighbor_edge;

	V_WORD	new_tris;
	new_tris.clear();

	V_WORD	vertices;

	V_WORD	reduced;
	V_WORD :: iterator r_end;

	// 1 array of WORDS for each vertex.  This is to form
	//  caps for areas where 3 or more vertices share the 
	//  same position.

	V_INT * stitch_pairs = new V_INT[ m_wNumVerts ];
	assert( stitch_pairs != NULL );


	V_WORD					neighbor_tris;
	std::vector< WORD * >	is_welded;		// corresponds to entries for neighbor_tris
	V_WORD					edge;

	WELD_VALUE				nweld;


	for( tri = 0; tri < m_wNumInd / 3; tri++ )
	{
		neighbor_tris.clear();
		is_welded.clear();
		edge.clear();

		if( m_pTriNeighbors[tri].ind1 != NO_NEIGHBOR )
		{
			neighbor_tris.push_back( m_pTriNeighbors[tri].ind1 );
			is_welded.push_back( &( m_pTriNeighbors[tri].welded1 ));
			edge.push_back( 0 );
		}
		else
			assert( false );

		if( m_pTriNeighbors[tri].ind2 != NO_NEIGHBOR )
		{
			neighbor_tris.push_back( m_pTriNeighbors[tri].ind2 );
			is_welded.push_back( &( m_pTriNeighbors[tri].welded2 ));
			edge.push_back( 1 );
		}
		else
			assert( false );

		if( m_pTriNeighbors[tri].ind3 != NO_NEIGHBOR )
		{
			neighbor_tris.push_back( m_pTriNeighbors[tri].ind3 );
			is_welded.push_back( &( m_pTriNeighbors[tri].welded3 ));
			edge.push_back( 2 );
		}
		else
			assert( false );


		if( neighbor_tris.size() != 3 )
		{
			m_bOpenHull = true;
			FDebug("**** Warning - object is open hull\n");
		}

		for( n=0; n < neighbor_tris.size(); n++ )
		{

			if( *( is_welded[n] ) == NOT_WELDED )
			{
				// add vertices we need to stitch
				// if n = 2 then use vertices n, 0
				last_edge_vertex = ( edge[n] + 1 ) % 3;

				vertices.clear();
				reduced.clear();

				vertices.push_back( m_pIndices[ tri*3 + edge[n] ] );
				vertices.push_back( m_pIndices[ tri*3 + last_edge_vertex ] );

				// Which edge of the neighbor is the base trianlge, tri,
				//	 attached to?
				neighbor_edge = 4;

				if		( tri == m_pTriNeighbors[ neighbor_tris[n] ].ind1 )
				{
					nweld = m_pTriNeighbors[ neighbor_tris[n] ].welded1;
					// make sure neighbor tri reports same un-welded value
//@					assert( nweld == NOT_WELDED_STITCHED ||
//@							nweld == NOT_WELDED );

					neighbor_edge = 0;
				}
				else if	( tri == m_pTriNeighbors[ neighbor_tris[n] ].ind2 )
				{
					nweld = m_pTriNeighbors[ neighbor_tris[n] ].welded2;
					// make sure neighbor tri reports same un-welded value
//@					assert( nweld == NOT_WELDED_STITCHED ||
//@							nweld == NOT_WELDED );

					neighbor_edge = 1;
				}
				else if	( tri == m_pTriNeighbors[ neighbor_tris[n] ].ind3 )
				{
					nweld = m_pTriNeighbors[ neighbor_tris[n] ].welded3;
					// make sure neighbor tri reports same un-welded value
//@					assert( nweld == NOT_WELDED_STITCHED ||
//@							nweld == NOT_WELDED );

					neighbor_edge = 2;
				}

				if( neighbor_edge < 0 || neighbor_edge > 2 )
				{
					FDebug("Error - can't find base tri location relative to neighbor!!\n");
					assert(false);
				}

				last_edge_vertex = ( neighbor_edge + 1 ) % 3;
				vertices.push_back( m_pIndices[ neighbor_tris[n]*3 + neighbor_edge ] );
				vertices.push_back( m_pIndices[ neighbor_tris[n]*3 + last_edge_vertex ] );

				// Some vertices pushed back above might refer to the 
				//  same vertex.  Need to eliminate duplicate entries, so
				//  make a vector, sort & eliminate duplicates

				for( i=0; i < vertices.size(); i++ )
				{
					reduced.push_back( vertices.at(i) );					
				}


				// Sort to prepare for unique() reduction
				sort( reduced.begin(), reduced.end() );

				// Eliminate duplicate vertex entries in order to count
				//  and see if we have 3 or 4 unique vertices along this edge
				// This determines whether to create 1 or 2 tris there.

				r_end = unique( reduced.begin(), reduced.end() );
				reduced.erase( r_end, reduced.end() );


				switch( reduced.size() )
				{
				case 3:
					// stitch new tri with appropriate winding order
					if( vertices.at(2) == vertices.at(1) )
					{
						new_tris.push_back( vertices.at(1) );
						new_tris.push_back( vertices.at(0) );
						new_tris.push_back( vertices.at(3) );

						// Add "stitch-pair" - This is a pointer 
						//  from one edge vertex to another, in 
						//  appropriate winding order for a triangle
						//  adjacent to the one we just added.

						stitch_pairs[ vertices.at(3)].push_back( vertices.at(0) );

					}
					else if( vertices.at(3) == vertices.at(0) )
					{
						new_tris.push_back( vertices.at(1) );
						new_tris.push_back( vertices.at(0) );
						new_tris.push_back( vertices.at(2) );

						stitch_pairs[ vertices.at(1)].push_back( vertices.at(2) );
					}
					else
					{
						FDebug("Error: bad degenerate vertex indices in stitching!\n");
						assert( false );
					}

					FDebug("Single tri stitched along an edge instead of a quad - One vert used twice\n");


					break;

				case 4:
					new_tris.push_back( vertices.at(1) );
					new_tris.push_back( vertices.at(0) );
					new_tris.push_back( vertices.at(2) );

					// Add "stitch-pair" - This is a pointer 
					//  from one edge vertex to another, in 
					//  appropriate winding order for a triangle
					//  adjacent to the one we just added.
					stitch_pairs[ vertices.at(1)].push_back( vertices.at(2) );

					new_tris.push_back( vertices.at(3) );
					new_tris.push_back( vertices.at(2) );
					new_tris.push_back( vertices.at(0) );

					stitch_pairs[ vertices.at(3)].push_back( vertices.at(0) );

					break;
				default:
//gj					FDebug("Bad # vertices found for quad creation!  %d\n", reduced.size() );
//gj					assert( false );
					break;

				}

				// Now mark the welded value as having been welded
				// Also mark the neighbor tris welded flag

				*( is_welded[n] ) = NOT_WELDED_STITCHED;

				switch( neighbor_edge )
				{
				case 0:
					m_pTriNeighbors[ neighbor_tris[n] ].welded1 = NOT_WELDED_STITCHED;
					break;
				case 1:
					m_pTriNeighbors[ neighbor_tris[n] ].welded2 = NOT_WELDED_STITCHED;
					break;
				case 2:
					m_pTriNeighbors[ neighbor_tris[n] ].welded3 = NOT_WELDED_STITCHED;
					break;

				default:
					FDebug("bad neighbor edge\n");
					assert( false );
				}

			}
		}
	}



	/////////////////////////////////////////////////////////
	// Knit caps where more than two vertices share the same
	//  position.
	//
	// These caps may not be needed - The object could be an
	//  open hull, but open only at POINTS (vertices), and
	//  never open along edges.
	// A point where > 2 vertices are located but with no 
	//  triangles between the points will never be extruded
	//  to open the hull.
	// If the object is "exploded" to move the vertices apart
	//  then these caps are needed.
	// Also, if the shadow volume vertices are moved inward
	//  along the normal in order to smooth the shadow intersection
	//  with the polygon model (to smooth the popping in and 
	//  out of shadow as the volume penetrates through), these
	//  caps are needed to correct for overlapping polys.  The
	//  cap will be flipped to the opposite facing & will correct
	//  for the overlapped regions.
	
	// Above, we built sets of "stitch pairs" for each edge where
	//  zero-area quads were created.  These pairs of vertices
	//  are the edges of those quads which are 1) not along the 
	//  original triangle edges, and 2) not the quad diagonal.
	// So, to make the caps, sort & search the stitch pairs.
	//  Wherever they form a closed loop, that's where we add
	//  a cap,  Winding order for the tris of this cap falls out
	//  of the order of verts in the stitch pair.

	// In most cases there will be only 1 entry for each vertex
	//  in the stitch pair.  If there are 2 entries, that means
	//  that TWO loops intersect.  One vertex points to two 
	//  appropriate verts that would be next in the loop.
	//@@@@  I don't handle that case in this code yet!  It just
	//  asserts false.
	
	
	V_WORD	ring;

	int		index, new_index;
	int		start_index;		// for sanity check
	#define MAX_IN_RING		20


	for( n=0; n < m_wNumVerts; n++ )
	{
		start_index = n;
		index = n;

		ring.clear();

		while( stitch_pairs[index].size() > 0 )
		{
			if( stitch_pairs[index].size() > 1 )
			{
				FDebug("\n");
				FDebug("Intersecting loops found!!  Yikes!  This case not coded up yet!!!\n");
				assert( false );
			}
			
			ring.push_back( index );

			new_index = stitch_pairs[index].at(0);	// get 1st vertex pointed to
													// This is next vertex in the loop

			// remove 1st element (1st vertex pointed to, for this
			//  base vertex)
			// In most cases this will cause the size to go to zero, so
			//  no further looping from there will occur

			stitch_pairs[index].erase( stitch_pairs[index].begin() );
			
			if( ring.size() > MAX_IN_RING )
			{
				assert( false );
			}

			index = new_index;
		}


		if( ring.size() > 0 )
		{
			// FDebug("Ring found of size:  %d\n", ring.size() );

			if( ring.size() > 1 )
			{
				// If we've made a loop, then index will be start_index
				//  and this value WILL NOT be repeated at the end of the loop
				assert( index == start_index );
				
				if( ring.size() > 2 )
				{
					// make tris!
					
					for( i=1; i < ring.size() - 1; i++ )
					{
						new_tris.push_back( ring.at(0)   );
						new_tris.push_back( ring.at(i)   );
						new_tris.push_back( ring.at(i+1) );
					}
				}
			}
		}
	}
	

//@@@@@@@@

	/////////////////////////////////////////////////////////////
	// Re-allocate index data and add new_tris to the end of the
	//  regular geometry.


	FDebug("Old tri count:  %d\n", m_wNumInd / 3  );

	m_wOriginalModelTriIndices = m_wNumInd;
	m_wLastTriIndex = m_wNumInd;


	if( new_tris.size() > 0 )
	{
		int  new_num_ind = new_tris.size() + m_wNumInd;

		if( new_num_ind < 65536 )
		{
			m_wOriginalModelTriIndices = m_wNumInd;

			AllocateMoreIndices( (WORD) new_num_ind );

			m_wLastTriIndex = m_wNumInd;

			assert( m_wLastTriIndex == ( m_wOriginalModelTriIndices + new_tris.size() ) );

			for( n = 0; n < new_tris.size(); n++ )
			{
				m_pIndices[ m_wOriginalModelTriIndices + n ] = new_tris.at(n);				
			}
		}
		else
		{
			FDebug("New object would have too many indices!!\n");
			FDebug("   Can't store them in our WORD data!!\n");
			assert( false );
			// Doesn't matter that temp data is not freed - that'll happen
			//  eventually
			return( E_FAIL );
		}
	}
	else
	{
		FDebug("No new tris to add!\n");
	}


	FDebug("New tri count:  %d\n\n", m_wNumInd / 3  );


	hr = FreeTemporaryData();
	ASSERT_IF_FAILED(hr);

	SAFE_ARRAY_DELETE( stitch_pairs );

	return( hr );
}





