/*********************************************************************NVMH2****
Path:  C:\Dev\devrel\Nv_sdk_4\Dx8\CommonSrc
File:  ShadowVolumeObject8.h

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:

  ***********

A class for building shadow volume geometry compatible with a vertex shader
program for extruding the volume.

This class builds a closed hull with potential zero-area triangles so that
the shadow volume extrusion will work properly.

The input object must be a closed hull with no T-junctions.

Use this object's SimpleObject8 interfaces to generate basic objects, then
create a ShadowVolumeVBObject8 (vertex buffer object) from it for fast rendering.

Creating the object calls WeldFacetedFaces() which will re-calculate vertex
normals if any faces were welded into the same "smoothing group."  To avoid this
recalculation, pass a threshold greater than 1.0f and no faces will be welded,
however a great deal of zero-area triangles will be created.


  ***********

******************************************************************************/



#ifndef __NV_SHADOWVOLUMEOBJECT8_H__
#define __NV_SHADOWVOLUMEOBJECT8_H__


#include "WingedEdgeObject8.h"


//	typedef std::vector< WORD > V_WORD;



class ShadowVolumeObject8 : public WingedEdgeObject8
{
private:
			// When drawing model use tris     0 to m_nOriginalModelTriIndices
			// When drawing shadow volume, use 0 to m_nLastTriIndex

				// Original model tris are from indices 0
				//  to m_nOriginalModelIndices
	WORD	m_wOriginalModelTriIndices;

				// zero-area tris for extruding shadow volume
				//   are from indices m_nOriginalModelTriIndices to
				//   m_nLastTriIndex;
	WORD	m_wLastTriIndex;

	virtual void 	PostInit();

public:

						// Will weld & not create zero-area triangles between
						//   faces whos normals are similar.  If the dot product
						//   of adjacent face normals is greater than weld_edge_threshold
						//   then no zero-area tris will be created there.  Instead,
						//   the tris will be guaranteed to use the same vertices so 
						//   that no breaking of the closed hull occurs when the 
						//   shadow volume extruding vertex shader is used.

	HRESULT			BuildShadowVolume( float weld_edge_threshold );



	virtual HRESULT Free();
	
	void	DrawShadowVolume( LPDIRECT3DDEVICE8  pDev );

	virtual void ExplodeFaces( float distance );	// move faces out along vector from
													//  center of object

	void	GetTriCounts( WORD * pNumTotalTris, WORD * pNumWithoutShadowVolTris );


	ShadowVolumeObject8();
	virtual ~ShadowVolumeObject8();
};




#endif		// __NV_SHADOWVOLUMEOBJECT8_H__