/*********************************************************************NVMH2****
Path:  C:\Dev\devrel\Nv_sdk_4\Dx8\CommonSrc
File:  ShadowVolumeVBObject8.cpp

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/


#include <assert.h>
#include "NV_Error.h"

//@#include "SimpleObject8.h"
//@#include "SimpleVBObject8.h"
#include "ShadowVolumeObject8.h"
#include "ShadowVolumeVBObject8.h"

#include <math.h>
#include <d3d8.h>
#include <d3dx8.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


#ifndef SAFE_RELEASE
	#define SAFE_RELEASE(p) { if(p != NULL) { (p)->Release(); (p)=NULL; } }
#endif

#ifndef ASSERT_IF_FAILED
	#define ASSERT_IF_FAILED( hres )	\
	{									\
		if( FAILED(hres) )				\
		   assert( false );				\
	}
#endif


//////////////////////////////////////
// Construction/destruction

ShadowVolumeVBObject8::ShadowVolumeVBObject8()
:SimpleVBObject8()
{
	m_wLastTriIndex = 0;
	m_wOriginalModelTriIndices = 0;
}


ShadowVolumeVBObject8::~ShadowVolumeVBObject8()
{
	// Base class destructor called automaticaly =)
}


//////////////////////////////////////



HRESULT ShadowVolumeVBObject8::CreateFromShadowVolumeObject( ShadowVolumeObject8 * pObj, LPDIRECT3DDEVICE8 pDev )
{
	assert( pObj != NULL );
	assert( pDev != NULL );
	assert( pObj->IsValid() );

	HRESULT hr = S_OK;

	hr = CreateFromSimpleObject( pObj, pDev );

	WORD	tot_tri, basetri;

	pObj->GetTriCounts( & tot_tri, & basetri );

	m_wLastTriIndex = tot_tri;
	m_wOriginalModelTriIndices = basetri;

	m_bIsValid = true;

	return( hr );
}



HRESULT ShadowVolumeVBObject8::DrawIndexRange( WORD start, WORD end )
{
	// values should be the start and stop position in the 
	//  index buffer.
	// Prim count = ( end - start ) / 3 for TRIANGLELIST

	assert( m_pD3DDev != NULL );
	assert( m_bIsValid == true );
	assert( end >= start );

	HRESULT hr = S_OK;

	if( end > start )
	{
		switch( m_PrimType )
		{
		case D3DPT_TRIANGLELIST:

			hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer, sizeof( SimpleObjVertex ) );
			ASSERT_IF_FAILED(hr);

			hr = m_pD3DDev->SetIndices( m_pIndexBuffer, 0 );
			ASSERT_IF_FAILED(hr);

			hr = m_pD3DDev->DrawIndexedPrimitive( m_PrimType, 0, m_wNumVerts, start, (end-start)/3 );
			ASSERT_IF_FAILED(hr);

			break;

		default:
			assert( false );
			hr = E_FAIL;

		}
	}

	return( hr );
}


HRESULT ShadowVolumeVBObject8::DrawBaseModel()
{
	// Draw tris up to m_wOriginalModelTriIndices;
	HRESULT hr;
	hr = DrawIndexRange( 0, m_wOriginalModelTriIndices );
	return( hr );
}

HRESULT ShadowVolumeVBObject8::DrawZeroAreaTris()
{
	// Draw only tris after the base model

	HRESULT hr = S_OK;

	hr = DrawIndexRange( m_wOriginalModelTriIndices, m_wLastTriIndex );

	return( hr );
}

HRESULT ShadowVolumeVBObject8::DrawAllTris()
{
	return( Draw() );	
}