/*********************************************************************NVMH2****
Path:  C:\DEV\devrel\NV_SDK_4\DX8\CommonSrc
File:  SimpleObject8.h

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/




#ifndef __SimpleObject8_H__
#define __SimpleObject8_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <d3d8.h>
#include <d3dx8.h>
#include <vector>

#include "SimpleObjectVertex.h"

//////////////////////////////////////////////////////////

typedef std::vector< WORD > V_WORD;


//////////////////////////////////////////////////////////


class SimpleObject8  
{
public:
	bool		m_bIsValid;

	SimpleObjVertex *	m_pVertices;
	WORD				m_wNumVerts;
	WORD			*	m_pIndices;
	WORD				m_wNumInd;

	D3DPRIMITIVETYPE	m_PrimType;



	void UnAllocate();	// Free all allocated memory
	void Allocate( WORD nvert, WORD nind );
	void AllocateMoreIndices( WORD nind );		// copies old indices into new array

	bool IsValid()  { return( m_bIsValid ); };

	void InitSphere( float radius, int nLat, int nLong );  // Sphere with n_ verts each direction
	void InitSphere( float radius, int nLat, int nLong,
						const D3DXVECTOR3 & axis1, 
						const D3DXVECTOR3 & axis2, 
						const D3DXVECTOR3 & axis3			);

	void InitRectangularBlock_Smooth( const D3DXVECTOR3 center,
									  DWORD color,
									  const D3DXVECTOR3 & axis1,
									  const D3DXVECTOR3 & axis2,
									  const D3DXVECTOR3 & axis3 );

	void InitRectangularBlock_Faceted( const D3DXVECTOR3 center,
									   DWORD color,
									   const D3DXVECTOR3 & axis1,
									   const D3DXVECTOR3 & axis2,
									   const D3DXVECTOR3 & axis3  );

	void InitSpiral( float width, float height, int n_sub_width,
					int n_sub_height, float n_twists );

	void InitSpiral(	const D3DXVECTOR3 & width_direction, 
						const D3DXVECTOR3 & height_direction,
						float width, float height, int n_sub_width, 
						int n_sub_height, float n_twists );


	void InitTorus( float InnerRadius, float OuterRadius, UINT Sides, UINT Rings );
	void InitTorus( float InnerRadius, float OuterRadius, UINT Sides, UINT Rings,
					float TexCoordUMax, float TexCoordVMax );

	void InitTorusWrapped( float InnerRadius, float OuterRadius, UINT Sides, UINT Rings );

	void InitTorus( float InnerRadius, float OuterRadius, UINT Sides, UINT Rings,
								const D3DXVECTOR3 & axis1,
								const D3DXVECTOR3 & axis2,
								const D3DXVECTOR3 & axis3	);


	void InitClone( SimpleObject8 * pObj );			// copy of the input object

	virtual void PostInit() { m_bIsValid = true; };				// If derived objs need to update things

	////////////////////////////////////////////
	void InitTest1_4QuadPlane(	const D3DXVECTOR3 & center,
								const D3DXVECTOR3 & axis1,
								const D3DXVECTOR3 & axis2  );
	
	////////////////////////////////////////////

	void AddPositionNoise( D3DXVECTOR3 vNoiseDir, float noise_magnitude );

	void AddPositionNoiseGrid( D3DXVECTOR3 vNoiseDir, float noise_magnitude,
								int grid_nx, int nsmooth );

	void MultiplyTexCoords( float ufactor, float vfactor );

	void TranslateXMinToZero();
	void Translate( float x, float y, float z );

	void SetVertexColors( DWORD ARGB_color );


	virtual void ReverseWinding();		// change winding for all triangles
	void		 ReverseNormals();		// invert all normals


	void ComputeObjectCenter( D3DXVECTOR3 * pCenter );
	void ComputeTriCenter( D3DXVECTOR3 * pResult, WORD tri_index );

	////////////////////////////////////////////

	void MakeFaceted();				// create degenerate verts for each triangle

	virtual void ExplodeFaces( float distance );	// move faces out along vector from
													//  center of object

	///////////////////////
	void	TestEliminateTrisWithoutTheseVertices( V_WORD & verts );

	void	ListVertIndicesUsed();

	///////////////////////
	// You must keep the pointer returned, and pass
	//  it back to the Free() funcs to free it up 
	void FreeDuplicatePositions( D3DXVECTOR3 ** pPos );
	void DuplicatePositions( D3DXVECTOR3 ** pPos, WORD * num_verts );

	void FreeDuplicateIndices( WORD ** pInd );
	void DuplicateIndices( WORD ** pInd, WORD * old_n_ind );
	///////////////////////


	void GenerateNormals();			// from vertex pos and triangle data
	void GenerateNormalsCCW();		// Generate them facing the oposite direction


	virtual void Draw( LPDIRECT3DDEVICE8  pDev );


	WORD	GetNumVerts()   { return( m_wNumVerts );	};
	WORD	GetNumIndices() { return( m_wNumInd );		};


	SimpleObject8();
	virtual ~SimpleObject8();
};


#endif   // ndef __SimpleObject8_H__

