/*********************************************************************NVMH2****
Path:  C:\Dev\devrel\Nv_sdk_4\Dx8\CommonSrc
File:  WingedEdgeObject8.h

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:

Code for generating a winged-edge type structure for geometry.  Edges are not 
tracked, rather, each triangle has up to three neighbors and params to describe
the connections to neighboring faces.


******************************************************************************/


#ifndef __NV_WINGEDEDGEOBJECT8_H__
#define __NV_WINGEDEDGEOBJECT8_H__

#include "SimpleObject8.h"
#include <vector>



typedef WORD		WELD_VALUE;

#define NOT_WELDED			0
#define IS_WELDED			1
#define NOT_WELDED_STITCHED	2

//////////////////////////////////////////
#define NO_NEIGHBOR		65535


/////////////////////////////////////////////////////////////////////


struct NeighborTriIndices
{
	WORD		ind1;		// Index to first neighbor triangle
							//  This neighbor tri should be on the side
							//  formed by the first and second vertices
							//  of the root triangle to which this struct
							//  belongs
							// Max value is m_wNumInd / 3 // 3 indices per tri
	WELD_VALUE	welded1;	// Is the edge between v0, v1 and ind1 triangle
							//  degenerate -- Are v0, v1 shared by the neighbor
							//  triangle?
	WORD		ind2;		// Neighbor adjacent to the v1, v2 points
	WELD_VALUE	welded2;

	WORD		ind3;		// Neighbor adjacent to v2, v0 points
	WELD_VALUE	welded3;

	D3DXVECTOR3	face_normal;
};


struct CollapsedSetEntry
{
	// Data member for 1 vertex
	// Used to create alternate mapping into smaller set of vertices
	//  when finding winged-edge connections

	WORD	IndexToOriginalVerts;	// Index into m_pVertices for the data
									//  so two verts originaly degenerate in position
									//  can point to the same vertex in m_pVertices

	WORD	CollapsedSetIndex;		// index value (array position) of the vertex
									//  in the collapsed set of vertices.
									// This is used to build a list of which 
									//  triangles each vertex is a part of
};


class WingedEdgeObject8 : public SimpleObject8
{
private:

	float	m_fPositionSqrdThreshold;		// for determining which vertices
											//  are degenerate in position

	bool	m_bWingedEdgeDataBuilt;

	CollapsedSetEntry	* m_pCollapsedVertexSet;	// each entry maps 1 m_pVertices vertex
													// to a location in the collapsed set
	WORD	m_wNumInCollapsedSet;

	WORD	* m_wTriVertexIndicesToCollapsedSet;

	NeighborTriIndices * m_pTriNeighbors;		// one NeighborTriIndices struct for
												// each tri in m_pIndices

	SimpleObject8	* m_pWingedEdgeMarkersObject;	// geometry which marks the winged-edge
													// connections.


	HRESULT AllocateData( WORD numverts, WORD numtris );

	void	BuildNonDegeneratePositions();
	void	BuildNonEdgeDegenerateTris();

	void	CompareTris( std::vector< WORD > & set1, std::vector< WORD > & set2, 
						 std::vector< WORD > & result );


	void	Warning_OpenHull();			// warning message about having an open hull
	void	Warning_SoloEdge();			// An edge has < 2 tris shared

	void	SetToConstructionValues();

public:

	bool			m_bMarkWeldedVertices;	// use only for diagnostic/debug

	bool			m_bOpenHull;
	bool			m_bSoloEdge;


	HRESULT			BuildWingedEdgeData();


							// Generate additional geometry in the
							//  m_pWingedEdgeMarkersObject to mark the 
							//  winged-edge topology connections.
							// Currently a crude visualization.
	SimpleObject8 ** GenerateWingedEdgeMarkers();

							// weld faces (collapse degenerate vertices
							//  and edges) if the dot product of the
							//  face normals is greater than threshold.
	void			WeldFacetedFaces( float threshold );


	virtual void	ReverseWinding();  

	
	virtual HRESULT Free();
	HRESULT			FreeTemporaryData();

	bool			IsWingedEdgeDataBuilt();

	WingedEdgeObject8();
	virtual ~WingedEdgeObject8();

	friend class ShadowVolumeObject8;
};




#endif		// __NV_WINGEDEDGEOBJECT8_H__