/*********************************************************************NVMH2****
Path:  E:\devrel\NV_SDK_4\DX8\NVEffectsBrowser\Effects\Explosions
File:  Explosions.cpp

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:
	Kenneth Hurley - Started on 4/07/2001.
	These particle effects use the software called Illusion 2.0.  You can
	find the software at http://www.coolfun.com

******************************************************************************/


#include "eb_effect.h"
#include "nvinc.h"
#include "Explosions.h"
#include "particles.h"
#include "D3dx8core.h"
#include <mmreg.h>
#include <dsound.h>
#include "dsutil.h"

// constant memory definitions for the vertex/pixel shader
#include "Constants.h"

#define VK_OEM_COMMA	0xbc
#define VK_OEM_PERIOD	0xbe

#define GRASS_HEIGHT (200.0f)
#define TREE_HEIGHT (2.2f)
#define DIRT_HEIGHT (2.85f)

char *LoadObjects[NUM_OBJECTS] = { "ccterrain.x",
								};

using namespace std;

bool gSyncToFrameRate = FALSE;
CSoundManager* g_pSoundManager = NULL;
CSound*        g_pSound = NULL;

DECLARE_EFFECT_MAIN()

char *strSingleStep = "Single Stepping Mode";

extern "C"
{
	
	__declspec(dllexport) unsigned int GetNumEffects() { return 1; }
	
	__declspec(dllexport) EBEffect* CreateEffect(unsigned int EffectNum)
	{	
		switch(EffectNum)
		{
		case 0:
			return new CExplosions();
			break;
		default:
			return NULL;
		}
	}
	
}

// ------------------------------------------------------------------------------
// CExplosions Constructor
//
// Description: Nothing to it -- just straight-forward construction
// ------------------------------------------------------------------------------ 
CExplosions::CExplosions()
:	m_bWireframe(false),
	m_mUI(NULL),
	m_pDecalTexture(NULL)
{
	DWORD i;
	m_strEffectLocation = "Particles\\Explosions";
	m_strEffectName = "Explosions";
	m_strEffectVertexShader = GetFilePath("particles.nvv");
	m_strEffectVersion = "1.0";
	m_strEffectPixelShader = "";
	
	m_TheParticleSystem = NULL;

	for (i=0; i<NUM_EXPLOSIONS; i++)
	{
		m_explosionsEmitter[i] = NULL;
	}

	D3DXMatrixIdentity(&m_MatWorld);
	D3DXMatrixIdentity(&m_MatView);
	D3DXMatrixIdentity(&m_MatProj);

	m_Stepping = false;

	for (i=0; i< NUM_OBJECTS; i++)
	{
		m_pVertexBuffer[i] = NULL;
		m_pIndexBuffer[i] = NULL;
		m_pAttributes[i] = NULL;
	}

	m_dwDirLightVShader = NULL;
}

// ------------------------------------------------------------------------------
// CExplosions Destructor
//
// Description: Nothing to it -- just straight-forward destruction
// ------------------------------------------------------------------------------ 
CExplosions::~CExplosions()
{
	Free(); 
}

// ------------------------------------------------------------------------------
// CExplosions Resize
//
// Description: This is a stupid hack, because the effects browser doesn't allow
//				us to get at the windows handle
// ------------------------------------------------------------------------------ 
void CExplosions::Resize(HWND hWnd, int w, int h, bool)
{
	HRESULT hr;
	EBEffect::Resize(hWnd, w, h, true);

	if (g_pSoundManager == NULL)
	{
		// Create a static IDirectSound in the CSound class.  
		// Set coop level to DSSCL_PRIORITY, and set primary buffer 
		// format to stereo, 22kHz and 16-bit output.
		g_pSoundManager = new CSoundManager();

		if( FAILED( hr = g_pSoundManager->Initialize(hWnd, DSSCL_PRIORITY, 2, 22050, 16 ) ) )
		{
			MessageBox( NULL, "Error initializing DirectSound.  Explosions will now exit.", 
								"Explosions", MB_OK | MB_ICONERROR );
			return;
		}

		g_pSound = NULL;
		// Load the wave file into a DirectSound buffer
		if( FAILED( hr = g_pSoundManager->Create( &g_pSound, (char *)(GetFilePath("explode.wav").c_str()), 0, GUID_NULL ) ) )
		{
			SAFE_DELETE(g_pSoundManager);
			MessageBox(NULL, "Error loading sound.  Explosions will now exit.", 
								"Explosions", MB_OK | MB_ICONERROR );
		}
	}

}


HRESULT CExplosions::LoadXFile(const char* fileName, const DWORD dwFVF, DWORD index)
{
	ID3DXMesh *tempMesh, *tempMeshFVF, *tempMeshOpt;
    LPBYTE pSrc;
	
	HRESULT hr = D3DXLoadMeshFromX(const_cast<char*>(fileName), D3DXMESH_SYSTEMMEM, m_pD3DDev, NULL, 
		(ID3DXBuffer**)&m_pMaterials[index], &m_dwNumSections[index], &tempMesh);

	LPDIRECT3DVERTEXBUFFER8 tempVB = NULL;
	tempMesh->GetVertexBuffer(&tempVB);
	tempVB->Lock( 0, 0, (BYTE**)&pSrc, 0 );
	// grab bound sphere, so we may initially scale our view.
	D3DXComputeBoundingSphere(pSrc, tempMesh->GetNumVertices(), dwFVF, &m_ObjectCenter[index], &m_ObjectRadius[index]);
	tempVB->Unlock();
	SAFE_RELEASE(tempVB);
	
	tempMesh->Optimize(D3DXMESHOPT_ATTRSORT, NULL, NULL, NULL, NULL, &tempMeshOpt);

	DWORD attribSize = m_dwNumSections[index];
	hr = tempMeshOpt->GetAttributeTable(NULL, &attribSize);
	
	m_pAttributes[index] = new D3DXATTRIBUTERANGE[attribSize];
	
	hr = tempMeshOpt->GetAttributeTable(m_pAttributes[index], &attribSize);

	//convert to our format, we have to do this last since the previous steps require reading from the 
	// IB or VB, and we're converting to writeonly
	hr = tempMeshOpt->CloneMeshFVF(D3DXMESH_WRITEONLY, dwFVF, m_pD3DDev, &tempMeshFVF);
	
	tempMeshFVF->GetVertexBuffer(&m_pVertexBuffer[index]);
	tempMeshFVF->GetIndexBuffer(&m_pIndexBuffer[index]);

	SAFE_RELEASE(tempMesh);
	SAFE_RELEASE(tempMeshFVF);
	SAFE_RELEASE(tempMeshOpt);
	
	return S_OK;
}

// ------------------------------------------------------------------------------
// CExplosions::UpdateProperties
//
// Description: This adds options to the menu (click on "File" or right-click
// on the display.
// ------------------------------------------------------------------------------ 
void CExplosions::UpdateProperties()
{
	EBEffect::UpdateProperties();
	EBProperty* pBoolProp;

	pBoolProp = new EBProperty(strSingleStep, OBJECT_MEMBER(m_Stepping), EBTYPE_BOOL_PROP, PROPFLAG_CALLBACK);
	AddProperty(pBoolProp);

	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Terrain Rendering", 
							GetFilePath("dirLight.nvv"), EBTYPE_STRING_PROP));
	m_pVertexShaderEnum->AddEnumerant(new EBEnumValue(m_pVertexShaderEnum, "Explosion Rendering",
							GetFilePath("particles.nvv"), EBTYPE_STRING_PROP));

    string aboutText = GetFilePath("NVEffectsExplained.htm");
    aboutText += "#Explosion";
	SetAboutInfo( NULL, _T("Explosions Demo"), _T(aboutText.c_str()));
	SetAboutInfo( NULL, _T("Kenneth Hurley - Developer Relations"), _T("http://www.nvidia.com/developer"));
	SetAboutInfo( NULL, _T("NVIDIA Corporation"), _T("http://www.nvidia.com"));

}

// ------------------------------------------------------------------------------
// CExplosions::PropertyUpdateCallback
//
// Description: Our properties were updated, set the correct shader to match
// ------------------------------------------------------------------------------ 
void CExplosions::PropertyUpdateCallback(const EBProperty* pProperty, bool bWritten)
{
	DWORD i;
    // if m_bShowShadow or mUseAlternateQuad changed we need to regenerate the volume
	if (pProperty->IsKindOf(EBTYPE_BOOL_PROP))
	{
		if (bWritten) 
        {
			if (!pProperty->GetPropertyName().compare(strSingleStep))
			{
				m_TheParticleSystem->SetStepMode(m_Stepping);
				for (i=0; i<NUM_EXPLOSIONS; i++)
				{
					m_TheParticleSystem->ResetParticleEmitter(m_explosionsEmitter[i]);
				}
			}
        }
	}
}
// ------------------------------------------------------------------------------
// CExplosions::ConfirmDevice
//
// Description: Performs caps-bit checking to make sure the selected device 
//		supports this demo.  In this particular case we check for nothing!
// ------------------------------------------------------------------------------ 
HRESULT CExplosions::ConfirmDevice(D3DCAPS8* pCaps, DWORD dwBehavior, D3DFORMAT Format)
{
	if (!(pCaps->TextureCaps & D3DPTEXTURECAPS_MIPMAP))
	{
		m_strLastError = "Device does not support mipmaps!";
		return E_FAIL;
	}

	if(pCaps->MaxSimultaneousTextures < 2)
	{
		m_strLastError = "Device does not support two simultaneous textures!";
		return E_FAIL;
	}

	if(D3DSHADER_VERSION_MAJOR(pCaps->VertexShaderVersion) < 1)
	{
		m_strLastError = "Device does not support vertex shaders!";
		return E_FAIL;
	}

//	if (!(pCaps->SrcBlendCaps & D3DPBLENDCAPS_SRCALPHA) || !(pCaps->SrcBlendCaps & D3DPBLENDCAPS_ONE))
//	{
//		return E_FAIL;
//	}

	return S_OK;
}

// ------------------------------------------------------------------------------
// CExplosions::Initialize
//
// Description: Initialize render-states, vertex shader and vertex buffers
// ------------------------------------------------------------------------------ 
HRESULT CExplosions::Initialize(IDirect3DDevice8* pDev)
{
	HRESULT hr;
	float tempX, tempY, tempZ;
	DWORD i;

	// get the D3D device 
	m_pD3DDev = pDev;
	pDev->AddRef();
	
	//initialize mouse UI
	RECT rect;
	rect.left = rect.top = 0;
	m_pD3DDev->GetViewport(&m_D3DViewPort);
	rect.bottom = m_D3DViewPort.Height;
	rect.right  = m_D3DViewPort.Width;
	m_mUI = new MouseUI((const RECT)rect, true);
    m_mUI->SetRotationalSensitivityFactor(1.0f);
	
    m_pD3DDev->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW ); //D3DCULL_none ); 

	////////////////////////////////////////////////////////
	// Load vertex shader for two-sided lighting

	vector<DWORD> Declaration;

	// Microsoft's D3DVSD_REG is not working properly, so use
	//  this STL vector method instead

	Declaration.clear();
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT3  )); // Pos
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT3  )); // Normal
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT2  )); // Texture
	Declaration.push_back(D3DVSD_END());

	hr = LoadAndCreateShader(GetFilePath("dirLight.vso"), &Declaration[0], 
			0, SHADERTYPE_VERTEX, &m_dwDirLightVShader);

	CHECKHRESULT(hr);
	////////////////////////////////////////////////////////


	hr = D3DXCreateTextureFromFile(m_pD3DDev, GetFilePath("ccTerrainMap.dds").c_str(), &m_pDecalTexture);
	CHECKHRESULT(hr);

	for (i=0; i< NUM_OBJECTS; i++)
	{
		hr = LoadXFile(GetFilePath(LoadObjects[i]).c_str(), TSVERTEX_FVF, i);
		CHECKHRESULT(hr);
	}

	SetupFixedCamera(0.0f, 20.0f, m_ObjectRadius[TERRAIN_OBJECT]);

	m_TheParticleSystem = new CParticleSystem(pDev);
	assert(m_TheParticleSystem != NULL);

	hr = m_TheParticleSystem->LoadParticle("explosion.particle", &m_explosionsEmitter[0], CV_PARTICLE_CONSTANTS);
	CHECKHRESULT(hr);

	tempX = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) * 
		(m_ObjectRadius[TERRAIN_OBJECT]/4.0f);
	tempY = (((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) + .5f) * 
		(m_ObjectRadius[TERRAIN_OBJECT]/10.0f);
	tempZ = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) * 
			(m_ObjectRadius[TERRAIN_OBJECT]/2.0f);

	D3DXMatrixTranslation(&m_explosionsMat[0], tempX, tempY, tempZ);

	for (i=1; i< NUM_EXPLOSIONS; i++)
	{
		m_explosionsEmitter[i] = m_explosionsEmitter[0];
		tempX = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) * 
			(m_ObjectRadius[TERRAIN_OBJECT]/4.0f);
		tempY = (((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) + .5f) * 
			(m_ObjectRadius[TERRAIN_OBJECT]/10.0f);
		tempZ = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) *
			(m_ObjectRadius[TERRAIN_OBJECT]/2.0f);
		D3DXMatrixTranslation(&m_explosionsMat[i], tempX, tempY, tempZ);
	}

	return S_OK;
}

void CExplosions::SetupFixedCamera(float x, float y, float radius)
{
	D3DXMATRIX temp;
	m_vEyePt = D3DXVECTOR3(x, y, -radius+100.0f);
	m_vLookatPt = D3DXVECTOR3( x, y-10.0f, 0.0f);
	m_vUp = D3DXVECTOR3( 0.0f, 1.0f, 0.0f);

    D3DXMatrixLookAtLH(&m_MatView, &m_vEyePt, &m_vLookatPt, &m_vUp);
    FLOAT fAspect = m_D3DViewPort.Width / (FLOAT)m_D3DViewPort.Height;
    D3DXMatrixPerspectiveFovLH( &m_MatProj, D3DX_PI/3, fAspect, 1.0f,
                                radius*200);
}


HRESULT CExplosions::SetVertexShaderGlobalConstants()
{
    // set up vertex shading constants to contain proper
    // transformation matrices etc.
    D3DXMATRIX matWorldView;
	D3DXMATRIX matWorldViewProj;
	D3DXMATRIX matWorldI;
	D3DXMATRIX matViewUI;
	D3DXMATRIX tempMat;

	m_pD3DDev->SetVertexShaderConstant(CV_WORLD_0, &m_MatWorld(0, 0), 4);

	D3DXMatrixInverse(&matWorldI, NULL, &m_MatWorld);

	D3DXMatrixMultiply(&tempMat, &m_mUI->GetRotationMatrix(), &m_mUI->GetTranslationMatrix());
	D3DXMatrixMultiply(&matViewUI, &m_MatView, &tempMat);
	D3DXMatrixMultiply(&matWorldView,     &m_MatWorld,     &matViewUI);
	D3DXMatrixMultiply(&matWorldViewProj, &matWorldView, &m_MatProj);
	
	// Write Projection to clip space
	D3DXMatrixTranspose(&matWorldViewProj, &matWorldViewProj);
	m_pD3DDev->SetVertexShaderConstant(CV_WORLDVIEWPROJ_0, &matWorldViewProj(0, 0), 4);

	// made this a vector 4 so, we wouldn't have to set the w part in the shader
	// set 12:00 noon time
	D3DXVECTOR4 vecLight(1.0f, 1.0f, -1.0f, 1.0f);
	// put light into object space
	D3DXVec4Transform(&vecLight, &vecLight, &matWorldI);
	D3DXVec4Normalize(&vecLight, &vecLight);

	m_pD3DDev->SetVertexShaderConstant(CVP_LIGHT_DIRECTION, &vecLight, 1);

	// made this a vector 4 so, we wouldn't have to set the w part in the shader
	D3DXVECTOR4 eyePos;
	// put eye into object space.
	D3DXVec3TransformCoord((D3DXVECTOR3*)&eyePos, &m_vEyePt, &matWorldI);
	eyePos.w = 1.0f;
	m_pD3DDev->SetVertexShaderConstant(CV_EYE_POSITION, &eyePos, 1);

	D3DXVECTOR4 convenience(0.0f, .5f, 1.0f, 2.0f);
	m_pD3DDev->SetVertexShaderConstant(CV_CONVENIENCE, &convenience, 1);

	return S_OK;
}

// ------------------------------------------------------------------------------
// CExplosions::Free
//
// Description: Called when we switch away from this demo.	Free up all the 
//		memory that is in use.
// ------------------------------------------------------------------------------ 
HRESULT CExplosions::Free()
{
	DWORD i;
	if (m_explosionsEmitter[0] != NULL)
	{
		m_TheParticleSystem->DeleteParticleEmitter(m_explosionsEmitter[0]);
		m_explosionsEmitter[0] = NULL;
	}

	SAFE_DELETE(m_mUI);
	SAFE_DELETE( g_pSound );
	SAFE_DELETE( g_pSoundManager );
	SAFE_DELETE(m_TheParticleSystem);

	SAFE_RELEASE(m_pDecalTexture);

	for (i=0; i<NUM_OBJECTS; i++)
	{
		SAFE_RELEASE(m_pVertexBuffer[i]);
		SAFE_RELEASE(m_pIndexBuffer[i]);
		SAFE_DELETE_ARRAY(m_pAttributes[i]);
	}

	if (m_pD3DDev != NULL)
	{
		if (m_dwDirLightVShader != NULL)
		{
			m_pD3DDev->DeleteVertexShader(m_dwDirLightVShader);
			m_dwDirLightVShader = NULL;
		}
	}


	SAFE_RELEASE(m_pD3DDev);
	
	return S_OK;
}

// ------------------------------------------------------------------------------
// CExplosions::Start
//
// Description: Called to reset
// ------------------------------------------------------------------------------ 
HRESULT CExplosions::Start()
{
	return S_OK;
}

// ------------------------------------------------------------------------------
// CExplosions::Tick
//
// Description: This function is called every frame.  Here is where we write
//		and update constants, clear the z- and back-buffer and render our 
//		primitives
// ------------------------------------------------------------------------------ 
HRESULT CExplosions::Tick(EBTimer* pTimer)
{
	HRESULT hr;
	D3DXMATRIX tempMat;
	DWORD i;

	m_pD3DDev->SetRenderState(D3DRS_FILLMODE, (m_bWireframe ? D3DFILL_WIREFRAME : D3DFILL_SOLID));
	m_pD3DDev->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x00004080, 1.0f, 0L);

	m_pD3DDev->SetTexture(0,(LPDIRECT3DBASETEXTURE8)m_pDecalTexture);

	m_pD3DDev->SetTextureStageState(0,D3DTSS_MAGFILTER,D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_MINFILTER,D3DTEXF_LINEAR);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_TEXTURETRANSFORMFLAGS,0);
	m_pD3DDev->SetTextureStageState(0,D3DTSS_TEXCOORDINDEX,D3DTSS_TCI_PASSTHRU | 0 );

	m_pD3DDev->SetRenderState(D3DRS_LIGHTING, TRUE);
	m_pD3DDev->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	m_pD3DDev->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESS);
	m_pD3DDev->SetRenderState(D3DRS_LOCALVIEWER, TRUE);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
    m_pD3DDev->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);

	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE);
	m_pD3DDev->SetRenderState(D3DRS_LIGHTING, FALSE);
    m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
    m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);   

	hr = m_pD3DDev->SetVertexShader(m_dwDirLightVShader);
	if (FAILED(hr))
		return hr;	

	D3DXMatrixIdentity(&m_MatWorld);

	SetVertexShaderGlobalConstants();
	// use this vertex buffer as our source for vertices
	hr = m_pD3DDev->SetStreamSource(0, m_pVertexBuffer[TERRAIN_OBJECT], sizeof(TSVertex));
	CHECKHRESULT(hr);
	//set index buffer
	hr = m_pD3DDev->SetIndices(m_pIndexBuffer[TERRAIN_OBJECT], 0);
	CHECKHRESULT(hr);

	for(i = 0; i < m_dwNumSections[TERRAIN_OBJECT]; i++)
	{
		hr = m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, m_pAttributes[TERRAIN_OBJECT][i].VertexStart,
			m_pAttributes[TERRAIN_OBJECT][i].VertexCount, m_pAttributes[TERRAIN_OBJECT][i].FaceStart*3, 
			m_pAttributes[TERRAIN_OBJECT][i].FaceCount);

		if (FAILED(hr))
			return hr;
	}


	for (i=0; i<NUM_EXPLOSIONS; i++)
	{
		m_MatWorld = m_explosionsMat[i];
		SetVertexShaderGlobalConstants();
		hr = m_TheParticleSystem->DrawParticleEmitter(m_explosionsEmitter[i]);
		if (FAILED(hr))
			return hr;
	}

	return D3D_OK;
}


void CExplosions::MouseButton(HWND hWnd, eButtonID button, bool bDown, int x, int y)
{
	if (button == MOUSE_LEFTBUTTON)
	{
		if(bDown)
		{
			m_mUI->OnLButtonDown(x, y);
		}
		else
		{
			m_mUI->OnLButtonUp(x, y);
		}
	}
	return;
}

void CExplosions::MouseMove(HWND hWnd, int x, int y)
{
	m_mUI->OnMouseMove(x, y);
	return;
}

void CExplosions::Keyboard(DWORD dwKey, UINT nFlags, bool bDown)
{
	DWORD i;
	float tempX, tempY, tempZ;
	SUBPARTICLELIST::iterator itSubParticles;

	eEBKeyAction Action = TranslateEffectKey(dwKey, nFlags, bDown);

	switch (Action)
	{

	case EB_HELP:
			::MessageBoxEx( NULL, " Help : F1 - Help \n\n"
								  " Home - Reset To Defaults \n\n"
								  " 'W' - Wireframe Toggle \n\n"
								  " '+' - Zoom in\n\n"
								  " '-' - Zoom out\n\n"
								  " Space - Toggle step mode\n\n"
								  " '>' - Step forward\n\n"
								  " '<' - Step backward\n\n"
								  " 1,2,3,4,5 - Toggles subParticle X\n\n"
								  " 'S' - Toggle sync to rendering framerate\t\n\n"
								  " 'Enter' - Restart Explosions\n\n",

				   "Help", MB_ICONINFORMATION | MB_TASKMODAL, MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ) );
        break;
	
	case EB_WIREFRAME:
		m_bWireframe = !m_bWireframe;
		m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
		break;

	case EB_RESET:
		m_mUI->Reset();
		m_dwEffectDirtyFlags |= EBEFFECT_DIRTY_PUBLICSTATE;
		break;

	case EB_ADD:
        m_mUI->Translate( 0.0f, 0.0f, .10f );
        break;

	case EB_SUBTRACT:
		m_mUI->Translate( 0.0f, 0.0f, -.10f );
		break;

	case EB_PAUSE:
		m_Stepping = !m_Stepping;
		m_TheParticleSystem->SetStepMode(m_Stepping);
		m_TheParticleSystem->ResetParticleEmitter(m_explosionsEmitter[0]);
		break;
	

	default:
		if ( bDown )
		{
			switch ( dwKey )
			  {
				case VK_LEFT :
				case VK_NUMPAD4 :
				{
					m_mUI->Translate( -1.0f, 0.0f, 0.0f );
					break;
				}

				case VK_RIGHT :
				case VK_NUMPAD6 :
				{
					m_mUI->Translate(  1.0f, 0.0f, 0.0f );
					break;
				}

				case VK_UP :
				case VK_NUMPAD8 :
				{
					m_mUI->Translate(  0.0f, 0.0f, -1.0f );
					break;
				}

				case VK_DOWN :
				case VK_NUMPAD2 :
				{
					m_mUI->Translate( 0.0f, 0.0f, 1.0f );
					break;
				}

				case VK_PRIOR :
				case VK_NUMPAD9 :
				{
					m_mUI->Translate(  0.0f, 1.0f, 0.0f );
					break;
				}
				case VK_NEXT :
				case VK_NUMPAD3 :
				{
					m_mUI->Translate(  0.0f, -1.0f, 0.0f );
					break;
				}

				case VK_OEM_PERIOD:
				{
					m_TheParticleSystem->Step(m_explosionsEmitter[0], true);
					break;	
				}

				case VK_OEM_COMMA:
				{
					m_TheParticleSystem->Step(m_explosionsEmitter[0], false);
					break;	
				}
				
				case VK_RETURN:
				{
					tempX = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) * 
						(m_ObjectRadius[TERRAIN_OBJECT]/4.0f);
					tempY = (((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) + .5f) * 
						(m_ObjectRadius[TERRAIN_OBJECT]/10.0f);
					tempZ = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) * 
						(m_ObjectRadius[TERRAIN_OBJECT]/2.0f);
					D3DXMatrixTranslation(&m_explosionsMat[0], tempX, tempY, tempZ);


					for (i=1; i< NUM_EXPLOSIONS; i++)
					{
						m_explosionsEmitter[i] = m_explosionsEmitter[0];
						tempX = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) * 
							(m_ObjectRadius[TERRAIN_OBJECT]/4.0f);
						tempY = (((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) + .5f) * 
							(m_ObjectRadius[TERRAIN_OBJECT]/10.0f);
						tempZ = ((2.0f * (((float)rand()) / (float) RAND_MAX)) - 1.0f) * 
							(m_ObjectRadius[TERRAIN_OBJECT]/2.0f);
						D3DXMatrixTranslation(&m_explosionsMat[i], tempX, tempY, tempZ);
					}

					m_TheParticleSystem->ResetParticleEmitter(m_explosionsEmitter[0]);
					break;
				}

				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				{
					m_TheParticleSystem->Toggle(m_explosionsEmitter[0], dwKey-'1');
					break;
				}

				case 'S':
					gSyncToFrameRate = !gSyncToFrameRate;
					break;

				default :
				{
					break;
				}
			}
		}
	}
	return;
}