/*****************************************************************************
Path:  E:\devrel\NV_SDK_4\DX8\NVEffectsBrowser\Effects\Particles.cpp
File:  Particles.cpp

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:
	Kenneth Hurley - Started on 4/07/2001.
	These particle effects use the software called Illusion 2.0.  You can
	find the software at http://www.coolfun.com

******************************************************************************/

#include "eb_effect.h"
#include "nvinc.h"
#include "explosions.h"
#include "particles.h"
#include <time.h>
#include <mmreg.h>
#include <dsound.h>
#include "DSUtil.h"

//#define DEBUGCREATEONEPARTICLE 1
// constant memory definitions for the vertex/pixel shader
#include "Constants.h"

extern bool gSyncToFrameRate;

PARSETABLE gParentKeyTable[] = {
	{ "PARAMETERS", PARENTPARMS }
};

#define NUM_PARENT_KEYS (sizeof(gParentKeyTable) / sizeof(PARSETABLE))

PARSETABLE gChildKeyTable[] = {
	{ "PARAMETERS", CHILDPARMS },
	{ "VARIATIONS", CHILDVARS },
	{ "LIFEPARAMETERS", CHILDLIFEPARMS }
};

#define NUM_CHILD_KEYS (sizeof(gChildKeyTable) / sizeof(PARSETABLE))

float	piTable[3][4] = {	D3DX_PI, 1.0f/2.0f, 2.0f*D3DX_PI, 1.0f/(2.0f*D3DX_PI),
							1.0f, -1.0f/2.0f, 1.0f/24.0f, -1.0f/720.0f,
							1.0f, -1.0f/6.0f, 1.0f/120.0f, -1.0f/5040.0f };

#define SIZEPITABLE (sizeof(piTable) / sizeof(float))

extern CSound	*g_pSound;

//=====================================================================
// Function:	CParticleSystem
// Description:	CParticleSystem Constructor
// Parameters:	maxParticles = number of particles that can be allocated
//				 for vertex buffer.  No more than this can be created
//				 by instancing.
// Returns:		.
//=====================================================================
CParticleSystem::CParticleSystem(IDirect3DDevice8* pDev)
{
	HRESULT hr;
	assert(pDev != NULL);
	m_pD3DDev = pDev;

	// Seed the random-number generator with current time so that
	// the numbers will be different every time we run.
	srand( (unsigned)time( NULL ) );

	vector<DWORD> Declaration;
	// we have to do this here since 
	// setup the stream for particles
	Declaration.push_back(D3DVSD_STREAM(0));
	Declaration.push_back(D3DVSD_REG(0, D3DVSDT_FLOAT4));
	Declaration.push_back(D3DVSD_REG(1, D3DVSDT_FLOAT4));
	Declaration.push_back(D3DVSD_REG(2, D3DVSDT_FLOAT4));
	Declaration.push_back(D3DVSD_REG(3, D3DVSDT_FLOAT4));
	Declaration.push_back(D3DVSD_END());

	m_StepMode = false;

	// load the vertex shader for particles
	m_dwCurrentVShader = NULL;
	hr = effect_api::LoadAndCreateShader(m_pD3DDev, effect_api::GetFilePath("particles.vso"), &Declaration[0], 
			0, SHADERTYPE_VERTEX, &m_dwCurrentVShader);


}

//=====================================================================
// Function:	~CParticleSystem
// Description:	CParticleSystem Destructor
// Parameters:	.
// Returns:		.
//=====================================================================
CParticleSystem::~CParticleSystem()
{
	PARTICLESLIST::iterator itParticles;

	for (itParticles = m_ParticleList.begin(); itParticles != m_ParticleList.end(); itParticles++)
	{
		// tell routine not to remove it from the list, because we are iterating it.
		DeleteParticleEmitter(*itParticles, false);
	}

	m_ParticleList.clear();

	if (m_dwCurrentVShader != NULL)
	{
		m_pD3DDev->DeleteVertexShader(m_dwCurrentVShader);
	}
}

//=====================================================================
// Function:	CreateParticleEmitter
// Description:	Creates the particle emitter (parent object)
// Parameters:	life = time slices this emitter stays alive
//				maxParticles = maximum particles for all of children
//					particles
//				particleParms = creation time base parameters.
// Returns:		PARTICLELIST & reference to created particle
//=====================================================================
PARTICLES *CParticleSystem::CreateParticleEmitter(DWORD maxParticles, 
												  PARENTPARTICLEPARMS *particleParms)
{
	HRESULT hr;
	PARTICLES *curParticles;

	curParticles = new PARTICLES;
	if (curParticles == NULL)
	{
		return NULL;
	}

	m_ParticleList.push_back(curParticles);

	curParticles->m_pVertexBuffer = NULL;
	curParticles->m_pIndexBuffer = NULL;

	// create static vertex buffer at load time
	hr = m_pD3DDev->CreateVertexBuffer((NUMQUADVERTS*sizeof(PVERTEX))*maxParticles, 
												D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC , 0, 
                                                D3DPOOL_DEFAULT, &curParticles->m_pVertexBuffer);
	if (FAILED(hr))
	{
		MessageBox(NULL, "Failed to create particle vertex buffer!", "ERROR", MB_OK);
		return NULL;
	}

	// create static vertex buffer at load time
	hr = m_pD3DDev->CreateIndexBuffer((NUMQUADINDICES*sizeof(WORD))*maxParticles, D3DUSAGE_WRITEONLY | D3DUSAGE_DYNAMIC,
												D3DFMT_INDEX16, D3DPOOL_DEFAULT, &curParticles->m_pIndexBuffer);
	if (FAILED(hr))
	{
		MessageBox(NULL, "Failed to create particle vertex buffer!", "ERROR", MB_OK);
		return NULL;
	}

	curParticles->m_ParticleParms = *particleParms;
	curParticles->m_MaxParticles = maxParticles;
	curParticles->m_CurrentIndex = 0;

    curParticles->m_StartTime.time = INVALID_TIME;

	return curParticles;
}

//=====================================================================
// Function:	DeleteParticleEmitter
// Description:	Deletes the Particle emitter from the list
// Parameters:	PARTICLES *emitter
// Returns:		S_OK if it was found and deleted
//=====================================================================
HRESULT CParticleSystem::DeleteParticleEmitter(PARTICLES *emitter, bool removeFromList)
{
	SUBPARTICLELIST::iterator itSubParticles;

	if (emitter->m_pIndexBuffer != NULL)
	{
		emitter->m_pIndexBuffer->Release();
		emitter->m_pIndexBuffer = NULL;
	}

	if (emitter->m_pVertexBuffer != NULL)
	{
		emitter->m_pVertexBuffer->Release();
		emitter->m_pVertexBuffer = NULL;
	}

	for (itSubParticles = emitter->m_Children.begin(); 
			itSubParticles != emitter->m_Children.end(); itSubParticles++)
	{
		if ((*itSubParticles)->m_Texture != NULL)
		{
			(*itSubParticles)->m_Texture->Release();
			(*itSubParticles)->m_Texture = NULL;
		}
		delete *itSubParticles;
	}

	emitter->m_Children.clear();

	if (removeFromList)
	{
		m_ParticleList.remove(emitter);
	}

	delete emitter;
	return S_OK;
}

//=====================================================================
// Function:	CreateParticleInstance
// Description:	Create a child object of parent particle object
// Parameters:	emitter = parent particle container
//				particleParms = creation time parameters
//				particleVars = creation time variation parameters
//				particleLifeVars = life time variable parameters
// Returns:		S_OK if it was created and no errors
//=====================================================================
HRESULT CParticleSystem::CreateParticleInstance(PARTICLES *emitter, CHILDPARTICLEPARMS *particleParms, 
								CHILDPARTICLEPARMS *particleVars, PARTICLELIFEVARS *particleLifeVars,
								DWORD constantIndex, LPDIRECT3DTEXTURE8 texture)
{
	float createNumber;
	DWORD dwNum;
	DWORD currentAvail;
	SUBPARTICLES *curSubParticles;
	DWORD i;
	float j;
	float frameCount;
	float sAmt, eAmt;				// starting and ending amounts
	float amtLerp;
	CONSTANTLIFEDATA *tempData;
	float lastVSolution;			// last velocity solution
	float lastWSolution;			// last weight solution
	float lastSSolution;			// last spin solution
	float v0;
	float a;
	float tv0;						// temporary velocity start time
	float tw0;						// temporary weight start time
	float ts0;						// temporary spin start time
	float keySeconds;
	float emitterScale;

	// create new particles structure
	curSubParticles = new SUBPARTICLES;
	if (curSubParticles == NULL)
	{
		return ERROR_NOT_ENOUGH_MEMORY;
	}

	// save off new particle in parent's children list
	emitter->m_Children.push_back(curSubParticles);

	// save off texture pointer
	curSubParticles->m_Texture = texture;
	curSubParticles->m_ParticleParms = *particleParms;
	curSubParticles->m_ParticleVars = *particleVars;
	curSubParticles->m_ConstantIndex = constantIndex;
	curSubParticles->m_ParticleLifeVars = *particleLifeVars;

	tempData = &curSubParticles->m_ParticleLifeData;


	emitterScale = min(particleParms->life / emitter->m_ParticleParms.life, 1.0f); 
	for (i=0; i <KEYPOINTS-1; i++)
	{
		// calculate keypoints in terms of emitters life
		if (particleLifeVars->DirectionTime[i] <= 1.0f)
		{
			tempData->DirectionTime[i] = particleLifeVars->DirectionTime[i] * emitterScale;
		}
		else
		{
			tempData->DirectionTime[i] = 2.0f;
		}

		if (particleLifeVars->SpinTime[i] <= 1.0f)
		{
			tempData->SpinTime[i] = particleLifeVars->SpinTime[i] * emitterScale;
		}
		else
		{
			tempData->SpinTime[i] = 2.0f;
		}

		if (particleLifeVars->VelocityTime[i] <= 1.0f)
		{
			tempData->VelocityTime[i] = particleLifeVars->VelocityTime[i] * emitterScale;
		}
		else
		{
			tempData->VelocityTime[i] = 2.0f;
		}
		if (particleLifeVars->WeightTime[i] <= 1.0f)
		{
			tempData->WeightTime[i] = particleLifeVars->WeightTime[i] * emitterScale;
		}
		else
		{
			tempData->WeightTime[i] = 2.0f;
		}

		if (particleLifeVars->ColorXParentTime[i] <= 1.0f)
		{
			tempData->ColorXParentTime[i] = particleLifeVars->ColorXParentTime[i] * emitterScale;
		}
		else
		{
			tempData->ColorXParentTime[i] = 2.0f;
		}

	}

	// copy Spin as non-formula data
	for (i=0; i< KEYPOINTS; i++)
	{
		tempData->Direction[i] = particleLifeVars->Direction[i];
		tempData->RGBA[i] = particleLifeVars->RGBA[i];
	}

	// V = Velocity, W = Weight, S = Spin
	lastVSolution = 0.0f;
	lastWSolution = 0.0f;
	lastSSolution = 0.0f;
	tv0 = 0.0f;
	tw0 = 0.0f;
	ts0 = 0.0f;
	// Now calculate starting velocity/acceleration for formula
	// X1 = new position
	// X0 = start position
	// v0 = initial velocity/gravity
	// a = acceleration
	// t = time 0..1.0
	// X1 = X0 + (v0 * t) + 1/2(a * t^2)
	// this allows us to solve for velocity + acceleration/(weight + gravity)
	
	for (i=0; i < KEYPOINTS-1; i++)
	{
		// calculate float of seconds that this key will last
		// = life in seconds * delta percentage specified for this time slice
		keySeconds = (emitter->m_ParticleParms.life * (tempData->VelocityTime[i] - tv0));

		// get initial velocity
		v0 = particleLifeVars->SpinVelocityWeight[i].z;
		// calculate acceleration over time period
		a = ((particleLifeVars->SpinVelocityWeight[i+1].z - v0) * emitter->m_ParticleParms.frameRate);
	
		// scale initial velocity by frameRate.
		v0 *= emitter->m_ParticleParms.frameRate;

		// save value adjusted for frameRate playback
		tempData->SpinABVelocityAB[i].z = v0;
		tempData->SpinABVelocityAB[i].w = a / keySeconds;
		tempData->VelocityCWeightABC[i].x = lastVSolution;
		// put back into 0..1 scale of emitters life
		keySeconds /= emitter->m_ParticleParms.life;
		lastVSolution = lastVSolution + ((v0 * keySeconds) + (.5f * (a * (keySeconds*keySeconds))));

		// get next start time
		tv0 = tempData->VelocityTime[i];

		// calculate float of seconds that this key will last
		// = life in seconds * delta percentage specified for this time slice
		keySeconds = (emitter->m_ParticleParms.life * (tempData->WeightTime[i] - tw0));

		// get initial weight
		v0 = particleLifeVars->SpinVelocityWeight[i].w;
		a = ((particleLifeVars->SpinVelocityWeight[i+1].w - v0) * emitter->m_ParticleParms.frameRate);
		// scale initial weight by frameRate.
		v0 *= emitter->m_ParticleParms.frameRate;

		// save value adjusted for frameRate playback
		tempData->VelocityCWeightABC[i].y = v0;
		tempData->VelocityCWeightABC[i].z = a / keySeconds;
		tempData->VelocityCWeightABC[i].w = lastWSolution;
		keySeconds /= emitter->m_ParticleParms.life;
		lastWSolution = lastWSolution + ((v0 * keySeconds) + (.5f * (a * (keySeconds*keySeconds))));

		// get next start time
		tw0 = tempData->WeightTime[i];

		// calculate float of rotations in 1 second that this key will last
		// = life in seconds * delta percentage specified for this time slice
		keySeconds = (emitter->m_ParticleParms.life * (tempData->SpinTime[i] - ts0));

		// get initial spin
		v0 = particleLifeVars->SpinVelocityWeight[i].x;
		a = ((particleLifeVars->SpinVelocityWeight[i+1].x - v0) * emitter->m_ParticleParms.frameRate);
	
		// scale initial velocity by frameRate.
		v0 *= emitter->m_ParticleParms.frameRate;

		// save value adjusted for frameRate playback
		tempData->SpinABVelocityAB[i].x = v0;
		tempData->SpinABVelocityAB[i].y = a / keySeconds;
		tempData->SpinC[i].x = lastSSolution;
		tempData->SpinC[i].y = particleParms->alignment;
		tempData->SpinC[i].z = 1.0f - particleParms->alignment;
		// put back into 0..1 scale of emitters life
		keySeconds /= emitter->m_ParticleParms.life;
		lastSSolution = lastSSolution + ((v0 * keySeconds) + (.5f * (a * (keySeconds*keySeconds))));

		// get next start time
		ts0 = tempData->SpinTime[i];

	}

	// calculate # particles available in buffer
	currentAvail = (emitter->m_MaxParticles - emitter->m_CurrentIndex);
	createNumber = 0.0f;

	// Calculate creation #
	for (i=0; i< KEYPOINTS-1; i++)
	{
		frameCount = emitter->m_ParticleParms.frameKey[i+1] - emitter->m_ParticleParms.frameKey[i];
		sAmt = (emitter->m_ParticleParms.numberPerSecond[i] * particleParms->number) / 
					emitter->m_ParticleParms.frameRate;
		eAmt = (emitter->m_ParticleParms.numberPerSecond[i+1] * particleParms->number) /
					emitter->m_ParticleParms.frameRate;
		amtLerp = (eAmt - sAmt) / frameCount;
		for (j=0.0f; j<frameCount; j += 1.0f)
		{
			createNumber += sAmt;
			sAmt += amtLerp;
		}
	}

	dwNum = (DWORD)createNumber;
#ifdef DEBUGCREATEONEPARTICLE
	dwNum = 1;
#endif
	
	// check to see if enough room in vertex buffer
	if (dwNum > currentAvail)
	{
		if (currentAvail == 0)
		{
			return ERROR_NOT_ENOUGH_MEMORY;
		}

		// nope, this is maximum that can be added
		dwNum = currentAvail;
	}

	// save off our index into the vertex buffer
	curSubParticles->m_Index = emitter->m_CurrentIndex;
	curSubParticles->m_Count = dwNum;
	curSubParticles->m_ConstantIndex = constantIndex;
	curSubParticles->m_Enabled = TRUE;

	// bump parents current index
	emitter->m_CurrentIndex += dwNum;

	return (ResetParticle(emitter, curSubParticles));


}

//=====================================================================
// Function:	ResetParticleEmitter
// Description:	Reset each child of a particle emitter
// Parameters:	.
// Returns:		D3D_OK if success
//=====================================================================
HRESULT CParticleSystem::ResetParticleEmitter(PARTICLES *emitter)
{
	HRESULT hr;
	SUBPARTICLELIST::iterator itSubParticles;
	for (itSubParticles = emitter->m_Children.begin(); 
			itSubParticles != emitter->m_Children.end(); itSubParticles++)
	{
		hr = ResetParticle(emitter, (*itSubParticles));
		if (FAILED(hr))
			return hr;
	}

	emitter->m_StartTime.time = INVALID_TIME;
	emitter->m_StepTime.time = 0;
	emitter->m_StepTime.millitm = 0;

	return hr;
}

//=====================================================================
// Function:	ResetParticle
// Description:	Reset the vertex buffer with new data
// Parameters:	.
// Returns:		S_OK if draw succeded
//=====================================================================
HRESULT CParticleSystem::ResetParticle(PARTICLES *emitter, SUBPARTICLES *curSubParticles)
{
	WORD curQuadIndex;
	DWORD i;
	float j;
	DWORD k;
	char errStr[1024];
	DWORD count;
	PVERTEX tempVertex;
	PVERTEX *curParticleVB;
	WORD *curParticleIB;
	HRESULT hr;
	CHILDPARTICLEPARMS *particleParms;
	CHILDPARTICLEPARMS *particleVars;
	CONSTANTLIFEDATA *particleLifeData;
	float frameCount;
	float sAmt, eAmt;				// starting and ending amounts
	float amtLerp;
	float createNumber;
	float curTime;
	DWORD lockFlags;
	DWORD prevNum;

	particleLifeData = &curSubParticles->m_ParticleLifeData;
	particleParms = &curSubParticles->m_ParticleParms;
	particleVars = &curSubParticles->m_ParticleVars;

	count = curSubParticles->m_Count;

	// start at beginning of Index Buffer
	curQuadIndex = curSubParticles->m_Index * NUMQUADVERTS;

	lockFlags = (curSubParticles->m_Index == 0) ? D3DLOCK_DISCARD : D3DLOCK_NOOVERWRITE;
	// lock vertex buffer,* 4 for # vertices in a quad
	hr = emitter->m_pVertexBuffer->Lock(curSubParticles->m_Index * NUMQUADVERTS * sizeof(PVERTEX), 
					(count * NUMQUADVERTS * sizeof(PVERTEX)), 
							(BYTE **)&curParticleVB, lockFlags );
	if (FAILED(hr))
	{
		char Buff[256];
		D3DXGetErrorString(hr, &Buff[0], 255);
		sprintf(errStr, "Unable to lock vertex buffer to fill it.\nD3DError: %d\n%s", hr, Buff);
		MessageBox(NULL, errStr, "ERROR", MB_OK);
		return hr;
	}

	// lock vertex buffer
	hr = emitter->m_pIndexBuffer->Lock((curSubParticles->m_Index * NUMQUADINDICES) * sizeof(WORD), 
							(count * NUMQUADINDICES * sizeof(WORD)), 
							(BYTE **)&curParticleIB, lockFlags );
	if (FAILED(hr))
	{
		MessageBox(NULL, "Unable to lock index buffer to fill it.\n", "ERROR", MB_OK);
		return hr;
	}

	createNumber = 0.0f;
	prevNum = 0;
	// Now fill in vertex data
	for (i=0; i< KEYPOINTS-1; i++)
	{
		// get delta frame count
		frameCount = emitter->m_ParticleParms.frameKey[i+1] - emitter->m_ParticleParms.frameKey[i];
		// get starting amount per second / framerate = amount per frame
		sAmt = (emitter->m_ParticleParms.numberPerSecond[i] * particleParms->number) / 
					emitter->m_ParticleParms.frameRate;
		// and ending amount per second / framerate = Amount per frame
		eAmt = (emitter->m_ParticleParms.numberPerSecond[i+1] * particleParms->number) /
					emitter->m_ParticleParms.frameRate;
		// now calculate linear interpolant value to go from start # to end # per frame
		amtLerp = (eAmt - sAmt) / frameCount;
		// calculate current time as starting frame / frame Rate
		curTime = emitter->m_ParticleParms.frameKey[i] / emitter->m_ParticleParms.frameRate;

		// now do for each time slice
		for (j=0.0f; j<frameCount; j += 1.0f)
		{
			createNumber += sAmt;
			
			// for amount released per time slice, create a quads
			for (k=0; k<(DWORD)(createNumber)-prevNum; k++)
			{
				// save start position in vertex buffer
				tempVertex.StartPosition = emitter->m_ParticleParms.basePosition;
				tempVertex.StartPosition.z += particleParms->basePosition.z;

				// now do velocity same way
				tempVertex.Velocity = emitter->m_ParticleParms.baseVelocity * particleParms->baseVelocity;
				if (emitter->m_ParticleParms.baseVelocity != 0.0f)
				{
					tempVertex.Velocity -= ((fmodf((float)rand() / (float) RAND_MAX, emitter->m_ParticleParms.baseVelocity)) 
												* particleVars->baseVelocity);
				}

				// Now do direction
				// now make a random variation based on 0.0 = no variation and 1.0 =
				// maximum variation, which is stored in particleVars->direction.xyz
				tempVertex.Direction.x = (2.0f * ((((float)rand()) / (float) RAND_MAX) * particleVars->baseDirection.x) - 1.0f);
				tempVertex.Direction.y = (2.0f * ((((float)rand()) / (float) RAND_MAX) * particleVars->baseDirection.y) - 1.0f);
				tempVertex.Direction.z = (2.0f * ((((float)rand()) / (float) RAND_MAX) * particleVars->baseDirection.z) - 1.0f);
				tempVertex.Direction += emitter->m_ParticleParms.baseDirection;
				D3DXVec3Normalize(&tempVertex.Direction, &tempVertex.Direction);

				// now do weight same way
				tempVertex.Weight = emitter->m_ParticleParms.baseWeight * particleParms->baseWeight;
				if (emitter->m_ParticleParms.baseWeight != 0.0f)
				{
					tempVertex.Weight -= (fmodf((float)rand() / (float) RAND_MAX, emitter->m_ParticleParms.baseWeight) *
												particleVars->baseWeight);
				}

				// now do weight same way
				tempVertex.Spin = emitter->m_ParticleParms.baseSpin * particleParms->baseSpin;
				if (emitter->m_ParticleParms.baseSpin != 0.0f)
				{
					tempVertex.Spin -= (fmodf((float)rand() / (float) RAND_MAX, emitter->m_ParticleParms.baseSpin) * 
										particleVars->baseSpin);
				}

				// align texture to direction, if alignment flag is set
				if (particleParms->alignment == FLOAT_TRUE)
				{
					// original texture is aligned pointing right, which
					// is normalized 2d vector 1,0.  Dot Product gives
					// cosine of angle between direction and this vector.  Since
					// 1, 0 is constant just take arc cosine of direction in x
					tempVertex.InitAlign = acosf(tempVertex.Direction.x);
					// if direction is pointing up then make negative angle
					if (tempVertex.Direction.y > 0)
					{
						tempVertex.InitAlign = -tempVertex.InitAlign;
					}
				}


				if (curTime > 0.0f)
				{
					// set time to start this particle
					tempVertex.StartTime = -(curTime / emitter->m_ParticleParms.life);
				}
				else
				{
					tempVertex.StartTime = 0.0f;
				}

				// setup index buffer for this quad
				*curParticleIB++ = curQuadIndex;
				*curParticleIB++ = curQuadIndex+1;
				*curParticleIB++ = curQuadIndex+2;
				*curParticleIB++ = curQuadIndex+2;
				*curParticleIB++ = curQuadIndex+1;
				*curParticleIB++ = curQuadIndex+3;
				// bump index to number of vertices in a quad
				curQuadIndex += NUMQUADVERTS;

				tempVertex.Texture0.x = 0.0f;
				tempVertex.Texture0.y = 0.0f;
				tempVertex.xOffset = -(particleParms->basePosition.x/2)*TEXTURESCALE;
				tempVertex.yOffset = (particleParms->basePosition.y/2)*TEXTURESCALE;
				*curParticleVB++ = tempVertex;

				tempVertex.Texture0.x = 1.0f;
				tempVertex.Texture0.y = 0.0f;
				tempVertex.xOffset = (particleParms->basePosition.x/2)*TEXTURESCALE;
				tempVertex.yOffset = (particleParms->basePosition.y/2)*TEXTURESCALE;
				*curParticleVB++ = tempVertex;

				tempVertex.Texture0.x = 0.0f;
				tempVertex.Texture0.y = 1.0f;
				tempVertex.xOffset = -(particleParms->basePosition.x/2)*TEXTURESCALE;
				tempVertex.yOffset = -(particleParms->basePosition.y/2)*TEXTURESCALE;
				*curParticleVB++ = tempVertex;

				tempVertex.Texture0.x = 1.0f;
				tempVertex.Texture0.y = 1.0f;
				tempVertex.xOffset = (particleParms->basePosition.x/2)*TEXTURESCALE;
				tempVertex.yOffset = -(particleParms->basePosition.y/2)*TEXTURESCALE;
				*curParticleVB++ = tempVertex;
			}

			curTime += 1.0f/emitter->m_ParticleParms.frameRate;
			prevNum = (DWORD)createNumber;
			sAmt += amtLerp;
		}
	}

	hr = emitter->m_pIndexBuffer->Unlock();
	if (FAILED(hr))
	{
		MessageBox(NULL, "Unable to unlock index buffer???.\n", "ERROR", MB_OK);
		return hr;
	}

	hr = emitter->m_pVertexBuffer->Unlock();
	if (FAILED(hr))
	{
		MessageBox(NULL, "Unable to unlock vertex buffer???.\n", "ERROR", MB_OK);
		return hr;
	}

	return hr;
}

//=====================================================================
// Function:	DrawParticleEmitter
// Description:	Draw particle emitter in the system
// Parameters:	particleEmitter = pointer to PARTICLES * for emitter
// Returns:		S_OK if draw succeded
//=====================================================================
HRESULT CParticleSystem::DrawParticleEmitter(PARTICLES *particleEmitter)
{
	PARTICLESLIST::iterator itParticles;
	SUBPARTICLELIST::iterator itSubParticles;
	SUBPARTICLES *curSubParticles;
	HRESULT hr;
	D3DXVECTOR4 tempVec;
	struct _timeb now;
	float periodocity;
	DWORD i;

	hr = m_pD3DDev->SetVertexShader(m_dwCurrentVShader);
	if (FAILED(hr))
		return hr;	

	// These could be redundant, but we may set states differently outside of code
	// anyway, it's only once per particle system render call
	// setup to use textures
//	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
//	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TFACTOR );
//  m_pD3DDev->SetRenderState(D3DRS_TEXTUREFACTOR, 0xffffffff);   

	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	m_pD3DDev->SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
    m_pD3DDev->SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);

	m_pD3DDev->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE); 	//D3DTOP_MODULATE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	m_pD3DDev->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    m_pD3DDev->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
	m_pD3DDev->SetTextureStageState(1, D3DTSS_ALPHAOP, D3DTOP_DISABLE);
	m_pD3DDev->SetRenderState(D3DRS_LIGHTING,	     FALSE );
    m_pD3DDev->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
    m_pD3DDev->SetRenderState(D3DRS_ZWRITEENABLE, false);   

	m_pD3DDev->SetRenderState( D3DRS_FOGENABLE,      FALSE );
    m_pD3DDev->SetRenderState( D3DRS_FOGTABLEMODE,   D3DFOG_NONE );
    m_pD3DDev->SetRenderState( D3DRS_FOGVERTEXMODE,  D3DFOG_NONE );
    m_pD3DDev->SetRenderState( D3DRS_RANGEFOGENABLE, FALSE );
    // add alpha's straight to achieve a "burn-out" effect
    m_pD3DDev->SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA);
    m_pD3DDev->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);

	hr = m_pD3DDev->SetVertexShaderConstant(CV_PI_DATA, piTable, SIZEPITABLE);
	if (FAILED(hr))
		return hr;

	// not initialized
	curSubParticles = NULL;
	for (itParticles = m_ParticleList.begin(); itParticles != m_ParticleList.end(); itParticles++)
	{
		particleEmitter = *itParticles;

		if (particleEmitter->m_StartTime.time == INVALID_TIME)
		{
			_ftime(&particleEmitter->m_StartTime);
			// start playing sound here.
			// first check if playing
			if (g_pSound != NULL)
			{
				if( g_pSound->IsSoundPlaying() )
				{
					// To pause, just stop the buffer, but don't reset the position
					if( g_pSound )
						g_pSound->Reset();
				}

				if( FAILED( hr = g_pSound->Play( 0, 0L ) ) )
					return hr;
			}
		}


		if (!m_StepMode && !gSyncToFrameRate)
		{
			_ftime(&now);
		}
		else
		{
			if (gSyncToFrameRate)
			{
				particleEmitter->m_StepTime.millitm += (DWORD)((1.0f/30.0f) * MILLISPERSECOND);
				if (particleEmitter->m_StepTime.millitm > MILLISPERSECOND)
				{
					particleEmitter->m_StepTime.time += (particleEmitter->m_StepTime.millitm / MILLISPERSECOND);
					particleEmitter->m_StepTime.millitm %= MILLISPERSECOND;
				}
			}

			now = particleEmitter->m_StartTime;
			now.millitm += particleEmitter->m_StepTime.millitm;
			if (now.millitm >= MILLISPERSECOND)
			{
				now.time += now.millitm / MILLISPERSECOND; 
				now.millitm %= MILLISPERSECOND;
			}
			now.time += particleEmitter->m_StepTime.time;
		}

		float kDifference(static_cast<float>(now.time - particleEmitter->m_StartTime.time));
		kDifference += static_cast<float>(0.001f * (now.millitm - particleEmitter->m_StartTime.millitm));


		// use this vertex buffer as our source for vertices
		hr = m_pD3DDev->SetStreamSource(0, particleEmitter->m_pVertexBuffer, sizeof(PVERTEX));
		if (FAILED(hr))
			return hr;	
		//set index buffer
		hr = m_pD3DDev->SetIndices(particleEmitter->m_pIndexBuffer, 0);
		if (FAILED(hr))
			return hr;	

		for (itSubParticles = particleEmitter->m_Children.begin(); 
				itSubParticles != particleEmitter->m_Children.end(); itSubParticles++)
		{
			// see if texture is going to be switched
			if ((curSubParticles == NULL) || (curSubParticles->m_Texture != (*itSubParticles)->m_Texture))
			{
				m_pD3DDev->SetTexture(0, (*itSubParticles)->m_Texture);
			}

			// grab pointer.
			curSubParticles = *itSubParticles;

			if (!curSubParticles->m_Enabled)
			{
				continue;
			}

			// slow system down...
			// make time 0..1.0 for periodocity
			periodocity = min(kDifference / particleEmitter->m_ParticleParms.life, 1.0f); // periodocity
			tempVec.x = periodocity;
			tempVec.y = 1.0f / particleEmitter->m_ParticleParms.life;

			hr = m_pD3DDev->SetVertexShaderConstant(CV_PERIODOCITY, &tempVec, 1);
			if (FAILED(hr))
				return hr;	

			// send in interpolants to vertex shader
			hr = m_pD3DDev->SetVertexShaderConstant(CV_PARTICLE_CONSTANTS, &curSubParticles->m_ParticleLifeData, 
														NUM_PARTICLELIFEDATA);
			if (FAILED(hr))
				return hr;

			// build delta times for vertex shader

			// starting lerp value is assumed to be 0.0f for time start
			tempVec.x = 0.0f;
			tempVec.y = 0.0f;
			// get initial start value, which is delta in first case
			tempVec.z = curSubParticles->m_ParticleLifeData.DirectionTime[0];
			tempVec.w = curSubParticles->m_ParticleLifeData.ColorXParentTime[0];

			i = 0;
			do 
			{
				if (tempVec.z != 0.0f)
				{
					tempVec.z = 1.0f / tempVec.z;
				}

				if (tempVec.w != 0.0f)
				{
					tempVec.w = 1.0f / tempVec.w;
				}

				// send in start times/ 1.0f/delta values for vertex shader
				hr = m_pD3DDev->SetVertexShaderConstant(CV_PARVAR_STIMES+i, tempVec, 1);
				if (FAILED(hr))
					return hr;

				tempVec.x = curSubParticles->m_ParticleLifeData.DirectionTime[i];
				tempVec.y = curSubParticles->m_ParticleLifeData.ColorXParentTime[i];

				tempVec.z = curSubParticles->m_ParticleLifeData.DirectionTime[i+1] -
					tempVec.x;
				tempVec.w = curSubParticles->m_ParticleLifeData.ColorXParentTime[i+1] -
					tempVec.y;

				i++;

			}
			while (i < KEYPOINTS-1);

			hr = m_pD3DDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, curSubParticles->m_Index * NUMQUADVERTS, 
				curSubParticles->m_Count * NUMQUADVERTS, curSubParticles->m_Index * NUMQUADINDICES, 
				curSubParticles->m_Count * NUMQUADTRIANGLES);

			if (FAILED(hr))
				return hr;
			
		}

	}

	return D3D_OK;
}

//=====================================================================
// Function:	ParseFloatArray
// Description:	parse a series of floats from a string
// Parameters:	parsePos = char *to parse from
//				dest = float * to store values into
//				curParmCount = pointer to dword that gets updated with
//				number of parmeters found in string
//				maxParms = maxiumum number of parameters in Array
// Returns:		S_OK if succeded, else error
//=====================================================================
HRESULT CParticleSystem::ParseFloatArray(char *parsePos, float *dest, 
										 DWORD *curParmCount, DWORD maxParms)
{
	DWORD sLen;
	char *endPos;
	HRESULT hr;
	float temp;
	DWORD color;

	sLen = strlen(parsePos);
	endPos = parsePos + sLen;

	while ((*curParmCount < maxParms) && (parsePos < endPos))
	{
		// first strip off whitespace if any
		parsePos += strspn(parsePos, " \t");

		// if comment OR EOL skip rest of line
		if (((*parsePos == '/') && (*(parsePos+1) == '/')) || (*parsePos == '\n'))

		{
			return S_OK;
		}

		// check if encoded value in DWORD
		if ((*parsePos == '0') && ((*(parsePos+1) == 'x') || (*(parsePos+1) == 'X')))
		{
			// scan hexidecimal numbers
			hr = sscanf(parsePos, "%x", &color);
			if (!hr)
			{
				return ERROR_BAD_FORMAT;
			}

			// make sure there is room in the output buffer
			if ((*curParmCount + 4) > maxParms)
			{
				return ERROR_INSUFFICIENT_BUFFER;
			}


			// store individual components as floats 0.0<->1.0
			*dest++ = (float)((color & 0xff000000) >> 24) / 255.0f;
			*dest++ = (float)((color & 0xff0000) >> 16) / 255.0f;
			*dest++ = (float)((color & 0xff00) >> 8) / 255.0f;
			*dest++ = (float)((color & 0xff) >> 0) / 255.0f;
			(*curParmCount) += 4;
		}
		else
		{
			// scan a floating point value.
			hr = sscanf(parsePos, "%f", &temp);
			if (!hr)
			{
				return ERROR_BAD_FORMAT;
			}

			*dest++ = temp;
			(*curParmCount)++;
		}

		parsePos += strcspn(parsePos, ",\n/");
		if (*parsePos == ',')
		{
			parsePos++;
		}
	}

	return S_OK;
}

//=====================================================================
// Function:	LoadParticle
// Description:	Load particle from text file
// Parameters:	filename = name of particle file to load
//				outEmitter pointer to receive pointer to PARTICLES structure
// Returns:		S_OK if draw succeded
//=====================================================================
HRESULT CParticleSystem::LoadParticle(char *fileName, PARTICLES **outEmitter, DWORD constantIndex)
{
	FILE *tFile;
	char line[MAX_INPUT_LINE];
	char errStr[256];
	char *parsePos;
	DWORD sLen;
	PARSESTATES curState;
	PARSESTATES nextState;
	char curName[MAX_NAME];
	DWORD curLine;
	DWORD curParmCount;
	DWORD childCount;
	DWORD curChildIndex;
	PARENTPARTICLEPARMS tempParentParms;
	CHILDPARTICLEPARMS tempChildParms;
	PARTICLELIFEVARS tempLifeVars;
	CHILDPARTICLEPARMS tempVars;
	char textureName[_MAX_PATH];
	HRESULT hr;
	DWORD childSections;
	DWORD numEntries;
	DWORD i;
	PARSETABLE *validKeyTable;
	PARTICLES *emitter;
	LPDIRECT3DTEXTURE8 pTexture;

	tFile = fopen(effect_api::GetFilePath(fileName).c_str(), "rt");

	if (tFile == NULL)
	{
		return ERROR_FILE_NOT_FOUND;
	}

	curState = NONE;
	nextState = NONE;
	curLine = 0;
	// go until end of file
	while (!feof(tFile))
	{
		if (fgets(line, MAX_INPUT_LINE, tFile) == NULL)
		{
			if (feof(tFile))
				break;
			return ferror(tFile);
		}

		curLine++;
		sLen = strlen(line);

		// strip off leading spaces.
		parsePos = line + strspn(line, " \t");

		// check if comment/ blank line
		if (((*parsePos == '/') && (*(parsePos+1) == '/')) || (*parsePos == '\n'))
			continue;

		// now make a keyword out of this
		switch (curState)
		{
		case DONE:
			break;

		case NONE:
		case CHILDNAME:
			childSections = 0;
			sLen = strcspn(parsePos, " \t\n");
			// after open brace, we should be getting parents parameters
			if (curState == NONE)
			{
				nextState = PARENTKEYWORD;
			}
			else
			{
				nextState = CHILDKEYWORD;
			}
			
			curState = OPENBRACE;
			if (sLen > MAX_NAME-1)
			{
				sLen = MAX_NAME-1;
			}
			strncpy(curName, parsePos, sLen);
			curName[sLen] = '\0';
			break;

		case PARENTKEYWORD:
		case CHILDKEYWORD:
			if (curState == PARENTKEYWORD)
			{
				validKeyTable = gParentKeyTable;
				numEntries = NUM_PARENT_KEYS;
			}
			else
			{
				validKeyTable = gChildKeyTable;
				numEntries = NUM_CHILD_KEYS;
			}

			nextState = curState;
			for (i=0; i< numEntries; i++)
			{
				sLen = strlen(validKeyTable->name);
				// check for keyword match
				if (!strnicmp(parsePos, validKeyTable->name, sLen))
				{
					nextState = validKeyTable->nextState;
					break;
				}
				validKeyTable++;
			}

			if (nextState == curState)
			{
				sprintf(errStr, "%s was not expected inside this block at line %d.\n", parsePos, curLine);
				MessageBox(NULL, errStr, "Parse Error", MB_OK);
				fclose(tFile);
				return ERROR_INVALID_BLOCK;
			}
			else if (curState == CHILDKEYWORD)
			{
				if (childSections & nextState)
				{
					sprintf(errStr, "More than one %s keyword section was found at line %d.\n", 
								validKeyTable->name, curLine);
					MessageBox(NULL, errStr, "Parse Error", MB_OK);
					fclose(tFile);
					return ERROR_INVALID_BLOCK;

				}

				childSections |= nextState;
			}


			curState = OPENBRACE;
			curParmCount = 0;
			break;

		case PARENTPARMS:
			sLen = 1;
			if (curParmCount == 0)
			{
				if (!sscanf(parsePos, "%d", &childCount) || (childCount == 0))
				{
					sprintf(errStr, "Child count cannot be zero or there was a parse error at line %d.\n", curLine);
					MessageBox(NULL, errStr, "Parse Error", MB_OK);
					fclose(tFile);
					return ERROR_BAD_FORMAT;
				}
				curChildIndex = 0;
				curParmCount++;

				parsePos += strcspn(parsePos, ",\n/");
				if (*parsePos != ',')
				{
					sLen = 0;
				}
				else
				{
					parsePos++;
				}
			}

			if (sLen != 0)
			{
				// parse floats into array
				hr = ParseFloatArray(parsePos, ((float *)&tempParentParms)+(curParmCount-1), 
								&curParmCount, NUM_PARENTPARTICLEPARMS+1);
				if (FAILED(hr))
				{
					sprintf(errStr, "Parse error at line %d, not enough parameters?.\n", curLine);
					MessageBox(NULL, errStr, "Parse Error", MB_OK);
					fclose(tFile);
					return hr;
				}
			}

			if (curParmCount == NUM_PARENTPARTICLEPARMS+1)
			{
				emitter = CreateParticleEmitter((childCount * tempParentParms.maxParticles), &tempParentParms);
				if (emitter == NULL)
				{
					fclose(tFile);
					return ERROR_NOT_ENOUGH_MEMORY;
				}
				curState = CHILDNAME;
			}


			break;
	
		case CHILDPARMS:
			sLen = 1;
			if (curParmCount == 0)
			{
				if (*parsePos == '"')
				{
					parsePos++;
				}
				else
				{
					sprintf(errStr, "Didn't find texture name or \" at line %d.\n", curLine);
					MessageBox(NULL, errStr, "Parse Error", MB_OK);
					fclose(tFile);
					return ERROR_BAD_FORMAT;
				}

				sLen = strcspn(parsePos, "\" \t,\n/");
				if (sLen > 0)
				{
					strncpy(textureName, parsePos, sLen);
					textureName[sLen] = '\0';
				}

				curParmCount++;

				parsePos += strcspn(parsePos, ",\n/");
				if (*parsePos != ',')
				{
					sLen = 0;
				}
				else
				{
					parsePos++;
				}
			}

			if (sLen != 0)
			{
				// parse floats into array
				hr = ParseFloatArray(parsePos, ((float *)&tempChildParms)+(curParmCount-1), 
								&curParmCount, NUM_CHILDPARTICLEPARMS+1);
				if (FAILED(hr))
				{
					sprintf(errStr, "Parse error at line %d, not enough parameters?.\n", curLine);
					MessageBox(NULL, errStr, "Parse Error", MB_OK);
					fclose(tFile);
					return hr;
				}
			}

			if (curParmCount == NUM_CHILDPARTICLEPARMS+1)
			{
				if ((childSections & CHILD_KEY_MASK) == CHILD_KEY_MASK)
				{
					nextState = EXITCHILD;
				}
				else
				{
					nextState = CHILDKEYWORD;

				}
				curState = CLOSEDBRACE;
			}

			break;

		case CHILDVARS:
			// parse floats into array
			hr = ParseFloatArray(parsePos, ((float *)&tempVars)+curParmCount, 
								&curParmCount, NUM_CHILDPARTICLEPARMS);
			if (FAILED(hr))
			{
				sprintf(errStr, "Parse error at or near %s at line %d.\n", parsePos, curLine);
				MessageBox(NULL, errStr, "Parse Error", MB_OK);
				fclose(tFile);
				return hr;
			}

			if (curParmCount == NUM_CHILDPARTICLEPARMS)
			{
				if ((childSections & CHILD_KEY_MASK) == CHILD_KEY_MASK)
				{
					nextState = EXITCHILD;
				}
				else
				{
					nextState = CHILDKEYWORD;

				}
				curState = CLOSEDBRACE;
			}

			break;

		case CHILDLIFEPARMS:
			// parse floats into array
			hr = ParseFloatArray(parsePos, ((float *)&tempLifeVars)+curParmCount, 
									&curParmCount, NUM_PARTICLELIFEVARS);
			if (FAILED(hr))
			{
				sprintf(errStr, "Parse error at or near %s at line %d.\n", parsePos, curLine);
				MessageBox(NULL, errStr, "Parse Error", MB_OK);
				fclose(tFile);
				return hr;
			}

			if (curParmCount == NUM_PARTICLELIFEVARS)
			{
				if ((childSections & CHILD_KEY_MASK) == CHILD_KEY_MASK)
				{
					nextState = EXITCHILD;
				}
				else
				{
					nextState = CHILDKEYWORD;

				}
				curState = CLOSEDBRACE;
			}
			break;

		case OPENBRACE:
			if (*parsePos != '{')
			{
				sprintf(errStr, "{ was expected at line %d.\n", curLine);
				MessageBox(NULL, errStr, "Parse Error", MB_OK);
				fclose(tFile);
				return ERROR_INVALID_BLOCK;
			}
			curState = nextState;
			nextState = NONE;
			break;

		case EXITCHILD:
			if (*parsePos != '}')
			{
				sprintf(errStr, "} was expected at line %d.\n", curLine);
				MessageBox(NULL, errStr, "Parse Error", MB_OK);
				fclose(tFile);
				return ERROR_INVALID_BLOCK;
			}
			childCount--;
			//load texture
			pTexture = NULL;
			if (textureName != NULL)
			{
				hr = D3DXCreateTextureFromFile(m_pD3DDev, effect_api::GetFilePath(textureName).c_str(), &pTexture );
				if (FAILED(hr))
				{
					char Buff[256];
					D3DXGetErrorString(hr, &Buff[0], 255);

					sprintf(errStr, "Error loading texture from file\nD3DError: %d\n%s", hr, Buff);
					MessageBox(NULL, errStr, "Load Error", MB_OK);
				}
			}

#ifdef DEBUGCREATEONEPARTICLE
			if (curChildIndex == 2)
			{
				hr = CreateParticleInstance(emitter, &tempChildParms, &tempVars, &tempLifeVars,
													constantIndex, pTexture);
			}
#else
			hr = CreateParticleInstance(emitter, &tempChildParms, &tempVars, &tempLifeVars,
												constantIndex, pTexture);
#endif
			if (FAILED(hr))
			{
				DeleteParticleEmitter(emitter);
				fclose(tFile);
				return hr;
			}

			// next child index
			curChildIndex++;

			if (childCount != 0)
			{
				curState = CHILDNAME;
			}
			else
			{
				curState = EXITPARENT;
			}
			break;

		case EXITPARENT:
			if (*parsePos != '}')
			{
				sprintf(errStr, "} was expected at line %d.\n", curLine);
				MessageBox(NULL, errStr, "Parse Error", MB_OK);
				fclose(tFile);
				return ERROR_INVALID_BLOCK;
			}
			curState = CLOSEDBRACE;
			nextState = DONE;
			break;

		case CLOSEDBRACE:
			if (*parsePos != '}')
			{
				sprintf(errStr, "} was expected at line %d.\n", curLine);
				MessageBox(NULL, errStr, "Parse Error", MB_OK);
				fclose(tFile);
				return ERROR_INVALID_BLOCK;
			}
			curState = nextState;
			nextState = NONE;
			break;
		}

	}

	// return emitter particle created.
	*outEmitter = emitter;

	fclose(tFile);
	return S_OK;

}

//=====================================================================
// Function:	SetStepMode
// Description:	Sets step mode
// Parameters:	stepMode = true = on, false = off
// Returns:		.
//=====================================================================
void CParticleSystem::SetStepMode(bool stepMode)
{
	m_StepMode = stepMode;
}

//=====================================================================
// Function:	Step
// Description:	Sets next step time
// Parameters:	emitter = pointer to emitter
//				forward = true, !true = backward
// Returns:		.
//=====================================================================
void CParticleSystem::Step(PARTICLES *emitter, bool forward)
{
	float time;
	
	time = 1.0f / emitter->m_ParticleParms.frameRate;

	// if enabled, keep adding to step time
	if (m_StepMode)
	{
		if (forward)
		{
			emitter->m_StepTime.millitm += (DWORD)(fmodf(time, 1.0f) * MILLISPERSECOND);
			if (emitter->m_StepTime.millitm >= MILLISPERSECOND)
			{
 				emitter->m_StepTime.time += emitter->m_StepTime.millitm / MILLISPERSECOND; 
				emitter->m_StepTime.millitm %= MILLISPERSECOND;
			}
			emitter->m_StepTime.time += (DWORD)time;
		}
		else
		{
			if ((emitter->m_StepTime.time > 0) || (emitter->m_StepTime.millitm > 0))
			{
				emitter->m_StepTime.millitm -= (DWORD)(fmodf(time, 1.0f) * MILLISPERSECOND);
				if (emitter->m_StepTime.millitm < 0)
				{
					emitter->m_StepTime.millitm = -emitter->m_StepTime.millitm;
 					emitter->m_StepTime.time -= (emitter->m_StepTime.millitm / MILLISPERSECOND) + 1; 
					emitter->m_StepTime.millitm = MILLISPERSECOND-(emitter->m_StepTime.millitm % MILLISPERSECOND);
				}
				emitter->m_StepTime.time -= (DWORD)time;
				if (emitter->m_StepTime.time < 0)
				{
					emitter->m_StepTime.time = 0;
					emitter->m_StepTime.time = 0;
				}
			}
		}
	}
}

//=====================================================================
// Function:	Toggle
// Description:	toggle one of the sub particles/off/on
// Parameters:	emitter = pointer to emitter
//				index = subparticle # to toggle
// Returns:		.
//=====================================================================
void CParticleSystem::Toggle(PARTICLES *emitter, DWORD index)
{
	DWORD i;
	SUBPARTICLELIST::iterator itSubParticles;
	i = 0;
	for (itSubParticles = emitter->m_Children.begin(); 
			itSubParticles != emitter->m_Children.end(); itSubParticles++)
	{
		if (i == index)
			break;
		i++;
	}

	// not found?
	if (i != index)
	{
		return;
	}

	(*itSubParticles)->m_Enabled = !(*itSubParticles)->m_Enabled;
}

//=====================================================================
// Function:	CloneParticleEmitter
// Description:	clones the particle emitter, but with new variables
// Parameters:	srcEmitter = pointer to emitter source
// Returns:		pointer to new emitter class
//=====================================================================
PARTICLES *CParticleSystem::CloneParticleEmitter(PARTICLES *srcEmitter)
{
	PARTICLES *destEmitter;
	SUBPARTICLELIST::iterator itSubParticles;
	SUBPARTICLES *curChild;
	HRESULT hr;

	// create new parent
	destEmitter = CreateParticleEmitter(srcEmitter->m_MaxParticles, &srcEmitter->m_ParticleParms);
	if (destEmitter == NULL)
	{
		return destEmitter;
	}

	// create each of the children from source
	for (itSubParticles = srcEmitter->m_Children.begin(); 
			itSubParticles != srcEmitter->m_Children.end(); itSubParticles++)
	{
		curChild = *itSubParticles;

		hr = CreateParticleInstance(destEmitter, &curChild->m_ParticleParms, &curChild->m_ParticleVars,
										&curChild->m_ParticleLifeVars, 0, curChild->m_Texture);
		if (hr != S_OK)
		{
			DeleteParticleEmitter(destEmitter, true);
			return NULL;
		}
	}

	return destEmitter;

}
