/*************************************************************************
Path:  E:\devrel\NV_SDK_4\DX8\NVEffectsBrowser\Effects\Particles
File:  Particles.h

Copyright (C) 1999, 2000 NVIDIA Corporation
This file is provided without support, instruction, or implied warranty of any
kind.  NVIDIA makes no guarantee of its fitness for a particular purpose and is
not liable under any circumstances for any damages or loss whatsoever arising
from the use or inability to use this file or items derived from it.

Comments:


******************************************************************************/


#ifndef PARTICLES_H
#define PARTICLES_H

using namespace std;
#include <list>
#include <sys/timeb.h>

#define MAXPARTICLES 2000
#define NUMQUADVERTS 4
#define NUMQUADINDICES 6
#define NUMQUADTRIANGLES 2
#define MAX_INPUT_LINE 1024
#define MAX_NAME 256
#define KEYPOINTS 5
#define NUM_PARENTPARTICLEPARMS (sizeof(PARENTPARTICLEPARMS) / sizeof(float))
#define NUM_CHILDPARTICLEPARMS (sizeof(CHILDPARTICLEPARMS) / sizeof(float))
#define NUM_PARTICLELIFEVARS (sizeof(PARTICLELIFEVARS) / sizeof(float))
#define NUM_PARTICLELIFEDATA (sizeof(CONSTANTLIFEDATA) / sizeof(D3DXVECTOR4))
#define FLOAT_TRUE	(1.0f)
#define MILLISPERSECOND (1000)
#define TEXTURESCALE (1.0f/8.0f)

#define INVALID_TIME 0

// this is defined for the maximum value that the XY portion of
// the floating point number will fit in.  i.e. the float will contain
//	6 bits of offset, 12 bits of y, 12 bits of x
#define MAX_SIZE_VALUE ((float)(4096.0f))

typedef	enum 
{
	NONE = 0,
	PARENTKEYWORD = 1,
	PARENTPARMS = 2,
	CHILDNAME = 4,
	CHILDKEYWORD = 8,
	CHILDPARMS = 16,
	CHILDVARS = 32,
	CHILDLIFEPARMS = 64,
	OPENBRACE = 128,
	CLOSEDBRACE = 256,
	EXITPARENT = 512,
	EXITCHILD = 1024,
	DONE = 2048
} PARSESTATES;

#define CHILD_KEY_MASK (CHILDPARMS | CHILDVARS | CHILDLIFEPARMS)

typedef struct tagPARSETABLE
{
	char *name;
	PARSESTATES nextState;
} PARSETABLE;

//  particle vertex data
typedef struct tagPVERTEX
{
	D3DXVECTOR3 StartPosition;		// starting position for particle
	float		Velocity;			// velocity of particle
	D3DXVECTOR3 Direction;			// direction of particle
	float		Weight;				// weight +. - of particle
	float		Spin;				// spin around camera plane
	float		xOffset;			// x offset from center
	float		yOffset;			// y offset from center
	float		StartTime;			// which time slice to start at
	float		InitAlign;			// Initial texture direction alignment
	float		dummy;				// pad
	D3DXVECTOR2 Texture0;			// texcoord for stage 0
} PVERTEX;

typedef struct tagPARTICLELIFEVARS
{
	float		DirectionTime[KEYPOINTS-1];		// time for lerp keypoints
	float		SpinTime[KEYPOINTS-1];			// spin keys
	float		VelocityTime[KEYPOINTS-1];		// velocity keys (time stamps)
	float		WeightTime[KEYPOINTS-1];		// weight key points
	float		ColorXParentTime[KEYPOINTS-1];	// ARGB, where alpha = transparency value, keys
	D3DXVECTOR4 Direction[KEYPOINTS];			// Direction vector
	D3DXVECTOR4	SpinVelocityWeight[KEYPOINTS];	// spin/velocity/Weight packed values
	D3DXVECTOR4	RGBA[KEYPOINTS];				// ARGB colors
} PARTICLELIFEVARS;

typedef struct tagCONSTANTLIFEDATA
{
	float		DirectionTime[KEYPOINTS-1];		// time for lerp keypoints
	float		SpinTime[KEYPOINTS-1];			// spin keys
	float		VelocityTime[KEYPOINTS-1];		// velocity keys (time stamps)
	float		WeightTime[KEYPOINTS-1];		// weight key points
	float		ColorXParentTime[KEYPOINTS-1];	// ARGB, where alpha = transparency value, keys
	D3DXVECTOR4 Direction[KEYPOINTS];			// Direction vector
	D3DXVECTOR4	SpinABVelocityAB[KEYPOINTS-1];	// Spin in xy v0,acceleration portion of velocity function 
												// in zw. zw in last component in not used.
	D3DXVECTOR4	VelocityCWeightABC[KEYPOINTS-1]; // previous calculation of velocity function in x
												//  Initial weight, gravity and previous weight in yzw
	D3DXVECTOR4	SpinC[KEYPOINTS-1];				// Spin's acceleration, YZW are unused right now
	D3DXVECTOR4	RGBA[KEYPOINTS];
} CONSTANTLIFEDATA;

typedef struct tagPARENTPARTICLEPARMS
{
	float maxParticles;				// maximum particles to create
	float frameRate;				// what frame rate to emit particles at
	float life;						// how many seconds this particle lasts
	float frameKey[KEYPOINTS];	// key points to spread particles per second over
	float numberPerSecond[KEYPOINTS];	// particles per second
	D3DXVECTOR3 basePosition;		// starting position in 3-space
	D3DXVECTOR3 baseDirection;		// initial direction
	float baseVelocity;				// overall velocity
	float baseWeight;
	float baseSpin;
	float baseVisibility;

} PARENTPARTICLEPARMS;

typedef struct tagCHILDPARTICLEPARMS
{
	float life;						// how many time slices this particle lasts
	float number;
	D3DXVECTOR3 basePosition;
	D3DXVECTOR3 baseDirection;
	float baseVelocity;
	float baseWeight;
	float baseSpin;
	float baseVisibility;
	float alignment;				// whether to align texture map with initial direction

} CHILDPARTICLEPARMS;

typedef struct tagSUBPARTICLES
{
	DWORD m_Index;					// particle index to do draw primitives at.
	DWORD m_Count;					// count of particles to render
	DWORD m_ConstantIndex;			// where to put the constants at DrawPrimitive time
	bool m_Enabled;					// particle is enabled???
	PARTICLELIFEVARS m_ParticleLifeVars; // original variable data in file format
	CONSTANTLIFEDATA m_ParticleLifeData; // particles life integral variables
	CHILDPARTICLEPARMS m_ParticleParms;	// Creation time paramters
	CHILDPARTICLEPARMS m_ParticleVars;	// Variations passed in constant memory
	LPDIRECT3DTEXTURE8 m_Texture;	// texture pointer
} SUBPARTICLES;

// STL class for linked list (child<->child<->child)
typedef list<SUBPARTICLES *> SUBPARTICLELIST;

// now really define the structure that contains the reference to the
// PARTICLELIST pointer.
typedef struct tagPARTICLES
{
	LPDIRECT3DVERTEXBUFFER8	m_pVertexBuffer; // particle system's vertex buffer.
	LPDIRECT3DINDEXBUFFER8	m_pIndexBuffer;	// particle system's index buffer.
	SUBPARTICLELIST m_Children;				// possible children if parent particle
	DWORD m_CurrentIndex;					// number of particle entries available
	DWORD m_MaxParticles;					// total of all children particles that is maximum
	PARENTPARTICLEPARMS m_ParticleParms;	// Creation time paramters
	struct _timeb m_StartTime;				// last time value for particles
	struct _timeb m_StepTime;				// how much to step if in step mode
} PARTICLES;

// STL class for a linked list (PARENT (emitter) <-> PARENT) list
typedef list<PARTICLES *> PARTICLESLIST;

class CParticleSystem
{
private:
	PARTICLESLIST m_ParticleList;			// linked list of particles
	IDirect3DDevice8 *m_pD3DDev;
	DWORD	m_dwCurrentVShader;				// handle for particles shader
	bool	m_StepMode;						// wether in step mode or not
public:
	CParticleSystem(IDirect3DDevice8* pDev);
	~CParticleSystem();

	PARTICLES *CreateParticleEmitter(DWORD maxParticles, PARENTPARTICLEPARMS *particleParms);

	void UpdateParticleTime(float time);
	HRESULT DeleteParticleEmitter(PARTICLES *emitter, bool removeFromList = true);
	HRESULT CreateParticleInstance(PARTICLES *emitter, CHILDPARTICLEPARMS *particleParms, 
								CHILDPARTICLEPARMS *particleVars, PARTICLELIFEVARS *particleLifeVars,
								DWORD constantIndex, LPDIRECT3DTEXTURE8 texture);

	HRESULT LoadParticle(char *fileName, PARTICLES **outEmitter, DWORD constantIndex);
	HRESULT DrawParticleEmitter(PARTICLES *particleEmitter);
	HRESULT ParseFloatArray(char *parsePos, float *dest, DWORD *curParmCount, DWORD maxParms);
	HRESULT CParticleSystem::ResetParticle(PARTICLES *emitter, SUBPARTICLES * curSubParticles);

	HRESULT CParticleSystem::ResetParticleEmitter(PARTICLES *emitter);
	void SetStepMode(bool stepMode = true);
	void Step(PARTICLES *emitter, bool forward = true);
	void Toggle(PARTICLES *emitter, DWORD index);
	PARTICLES *CloneParticleEmitter(PARTICLES *srcEmitter);

};


#endif